#
# Copyright (c) 2024, Massachusetts Institute of Technology All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
#
# Redistributions in binary form must reproduce the above copyright notice, this
# list of conditions and the following disclaimer in the documentation and/or
# other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

EXCEPTION_MAP = {}
EXCEPTION_PREFIX_MAP = {}

def STATUS_OK(status):
    return (status & 1)

def STATUS_NOT_OK(status):
    return not (status & 1)

def STATUS_FACILITY(status):
    return (status >> 16)

def STATUS_MESSAGE(status):
    return (status >> 3) & 0b1111111111111

def STATUS_SEVERITY(status):
    return (status & 0b111)

def getException(status):
    return EXCEPTION_MAP.get(status, MdsException(f'Unknown status: {status}'))

def getExceptionFromError(error):
    if error is None:
        return None
    
    if error.startswith('%'):
        prefix = error.split(',', maxsplit=1)[0]
        return EXCEPTION_PREFIX_MAP.get(prefix, MdsException(error))

    return MdsException(error)

class MdsException(Exception):
    pass

class ServerException(MdsException):
    pass

class ServerNOT_DISPATCHED(ServerException):
    status = 266436616
    prefix = "%SERVER-W-NOT_DISPATCHED"
    def __init__(self):
        Exception.__init__(self, "%SERVER-W-NOT_DISPATCHED, Action not dispatched, depended on failed action")

EXCEPTION_MAP[ServerNOT_DISPATCHED.status] = ServerNOT_DISPATCHED
EXCEPTION_PREFIX_MAP[ServerNOT_DISPATCHED.prefix] = ServerNOT_DISPATCHED

class ServerINVALID_DEPENDENCY(ServerException):
    status = 266436626
    prefix = "%SERVER-E-INVALID_DEPENDENCY"
    def __init__(self):
        Exception.__init__(self, "%SERVER-E-INVALID_DEPENDENCY, Action dependency cannot be evaluated")

EXCEPTION_MAP[ServerINVALID_DEPENDENCY.status] = ServerINVALID_DEPENDENCY
EXCEPTION_PREFIX_MAP[ServerINVALID_DEPENDENCY.prefix] = ServerINVALID_DEPENDENCY

class ServerCANT_HAPPEN(ServerException):
    status = 266436634
    prefix = "%SERVER-E-CANT_HAPPEN"
    def __init__(self):
        Exception.__init__(self, "%SERVER-E-CANT_HAPPEN, Action contains circular dependency or depends on action which was not dispatched")

EXCEPTION_MAP[ServerCANT_HAPPEN.status] = ServerCANT_HAPPEN
EXCEPTION_PREFIX_MAP[ServerCANT_HAPPEN.prefix] = ServerCANT_HAPPEN

class ServerINVSHOT(ServerException):
    status = 266436642
    prefix = "%SERVER-E-INVSHOT"
    def __init__(self):
        Exception.__init__(self, "%SERVER-E-INVSHOT, Invalid shot number, cannot dispatch actions in model")

EXCEPTION_MAP[ServerINVSHOT.status] = ServerINVSHOT
EXCEPTION_PREFIX_MAP[ServerINVSHOT.prefix] = ServerINVSHOT

class ServerABORT(ServerException):
    status = 266436658
    prefix = "%SERVER-E-ABORT"
    def __init__(self):
        Exception.__init__(self, "%SERVER-E-ABORT, Server action was aborted")

EXCEPTION_MAP[ServerABORT.status] = ServerABORT
EXCEPTION_PREFIX_MAP[ServerABORT.prefix] = ServerABORT

class ServerPATH_DOWN(ServerException):
    status = 266436674
    prefix = "%SERVER-E-PATH_DOWN"
    def __init__(self):
        Exception.__init__(self, "%SERVER-E-PATH_DOWN, Path to server lost")

EXCEPTION_MAP[ServerPATH_DOWN.status] = ServerPATH_DOWN
EXCEPTION_PREFIX_MAP[ServerPATH_DOWN.prefix] = ServerPATH_DOWN

class ServerSOCKET_ADDR_ERROR(ServerException):
    status = 266436682
    prefix = "%SERVER-E-SOCKET_ADDR_ERROR"
    def __init__(self):
        Exception.__init__(self, "%SERVER-E-SOCKET_ADDR_ERROR, Cannot obtain ip address socket is bound to")

EXCEPTION_MAP[ServerSOCKET_ADDR_ERROR.status] = ServerSOCKET_ADDR_ERROR
EXCEPTION_PREFIX_MAP[ServerSOCKET_ADDR_ERROR.prefix] = ServerSOCKET_ADDR_ERROR

class ServerINVALID_ACTION_OPERATION(ServerException):
    status = 266436690
    prefix = "%SERVER-E-INVALID_ACTION_OPERATION"
    def __init__(self):
        Exception.__init__(self, "%SERVER-E-INVALID_ACTION_OPERATION, Unknown action operation")

EXCEPTION_MAP[ServerINVALID_ACTION_OPERATION.status] = ServerINVALID_ACTION_OPERATION
EXCEPTION_PREFIX_MAP[ServerINVALID_ACTION_OPERATION.prefix] = ServerINVALID_ACTION_OPERATION

class DevException(MdsException):
    pass

class DevBAD_ENDIDX(DevException):
    status = 662470666
    prefix = "%DEV-E-BAD_ENDIDX"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_ENDIDX, Unable to read end index for channel")

EXCEPTION_MAP[DevBAD_ENDIDX.status] = DevBAD_ENDIDX
EXCEPTION_PREFIX_MAP[DevBAD_ENDIDX.prefix] = DevBAD_ENDIDX

class DevBAD_FILTER(DevException):
    status = 662470674
    prefix = "%DEV-E-BAD_FILTER"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_FILTER, Illegal filter selected")

EXCEPTION_MAP[DevBAD_FILTER.status] = DevBAD_FILTER
EXCEPTION_PREFIX_MAP[DevBAD_FILTER.prefix] = DevBAD_FILTER

class DevBAD_FREQ(DevException):
    status = 662470682
    prefix = "%DEV-E-BAD_FREQ"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_FREQ, Illegal digitization frequency selected")

EXCEPTION_MAP[DevBAD_FREQ.status] = DevBAD_FREQ
EXCEPTION_PREFIX_MAP[DevBAD_FREQ.prefix] = DevBAD_FREQ

class DevBAD_GAIN(DevException):
    status = 662470690
    prefix = "%DEV-E-BAD_GAIN"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_GAIN, Illegal gain selected")

EXCEPTION_MAP[DevBAD_GAIN.status] = DevBAD_GAIN
EXCEPTION_PREFIX_MAP[DevBAD_GAIN.prefix] = DevBAD_GAIN

class DevBAD_HEADER(DevException):
    status = 662470698
    prefix = "%DEV-E-BAD_HEADER"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_HEADER, Unable to read header selection")

EXCEPTION_MAP[DevBAD_HEADER.status] = DevBAD_HEADER
EXCEPTION_PREFIX_MAP[DevBAD_HEADER.prefix] = DevBAD_HEADER

class DevBAD_HEADER_IDX(DevException):
    status = 662470706
    prefix = "%DEV-E-BAD_HEADER_IDX"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_HEADER_IDX, Unknown header configuration index")

EXCEPTION_MAP[DevBAD_HEADER_IDX.status] = DevBAD_HEADER_IDX
EXCEPTION_PREFIX_MAP[DevBAD_HEADER_IDX.prefix] = DevBAD_HEADER_IDX

class DevBAD_MEMORIES(DevException):
    status = 662470714
    prefix = "%DEV-E-BAD_MEMORIES"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_MEMORIES, Unable to read number of memory modules")

EXCEPTION_MAP[DevBAD_MEMORIES.status] = DevBAD_MEMORIES
EXCEPTION_PREFIX_MAP[DevBAD_MEMORIES.prefix] = DevBAD_MEMORIES

class DevBAD_MODE(DevException):
    status = 662470722
    prefix = "%DEV-E-BAD_MODE"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_MODE, Illegal mode selected")

EXCEPTION_MAP[DevBAD_MODE.status] = DevBAD_MODE
EXCEPTION_PREFIX_MAP[DevBAD_MODE.prefix] = DevBAD_MODE

class DevBAD_NAME(DevException):
    status = 662470730
    prefix = "%DEV-E-BAD_NAME"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_NAME, Unable to read module name")

EXCEPTION_MAP[DevBAD_NAME.status] = DevBAD_NAME
EXCEPTION_PREFIX_MAP[DevBAD_NAME.prefix] = DevBAD_NAME

class DevBAD_OFFSET(DevException):
    status = 662470738
    prefix = "%DEV-E-BAD_OFFSET"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_OFFSET, Illegal offset selected")

EXCEPTION_MAP[DevBAD_OFFSET.status] = DevBAD_OFFSET
EXCEPTION_PREFIX_MAP[DevBAD_OFFSET.prefix] = DevBAD_OFFSET

class DevBAD_STARTIDX(DevException):
    status = 662470746
    prefix = "%DEV-E-BAD_STARTIDX"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_STARTIDX, Unable to read start index for channel")

EXCEPTION_MAP[DevBAD_STARTIDX.status] = DevBAD_STARTIDX
EXCEPTION_PREFIX_MAP[DevBAD_STARTIDX.prefix] = DevBAD_STARTIDX

class DevNOT_TRIGGERED(DevException):
    status = 662470754
    prefix = "%DEV-E-NOT_TRIGGERED"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-NOT_TRIGGERED, Device was not triggered, check wires and triggering device")

EXCEPTION_MAP[DevNOT_TRIGGERED.status] = DevNOT_TRIGGERED
EXCEPTION_PREFIX_MAP[DevNOT_TRIGGERED.prefix] = DevNOT_TRIGGERED

class DevFREQ_TOO_HIGH(DevException):
    status = 662470762
    prefix = "%DEV-E-FREQ_TOO_HIGH"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-FREQ_TOO_HIGH, The frequency is set to high for the requested number of channels")

EXCEPTION_MAP[DevFREQ_TOO_HIGH.status] = DevFREQ_TOO_HIGH
EXCEPTION_PREFIX_MAP[DevFREQ_TOO_HIGH.prefix] = DevFREQ_TOO_HIGH

class DevINVALID_NOC(DevException):
    status = 662470770
    prefix = "%DEV-E-INVALID_NOC"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-INVALID_NOC, The NOC (number of channels) requested is greater than the physical number of channels")

EXCEPTION_MAP[DevINVALID_NOC.status] = DevINVALID_NOC
EXCEPTION_PREFIX_MAP[DevINVALID_NOC.prefix] = DevINVALID_NOC

class DevRANGE_MISMATCH(DevException):
    status = 662470778
    prefix = "%DEV-E-RANGE_MISMATCH"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-RANGE_MISMATCH, The range specified on the menu doesn't match the range setting on the device")

EXCEPTION_MAP[DevRANGE_MISMATCH.status] = DevRANGE_MISMATCH
EXCEPTION_PREFIX_MAP[DevRANGE_MISMATCH.prefix] = DevRANGE_MISMATCH

class DevBAD_VERBS(DevException):
    status = 662470794
    prefix = "%DEV-E-BAD_VERBS"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_VERBS, Error reading interpreter list (:VERBS)")

EXCEPTION_MAP[DevBAD_VERBS.status] = DevBAD_VERBS
EXCEPTION_PREFIX_MAP[DevBAD_VERBS.prefix] = DevBAD_VERBS

class DevBAD_COMMANDS(DevException):
    status = 662470802
    prefix = "%DEV-E-BAD_COMMANDS"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_COMMANDS, Error reading command list")

EXCEPTION_MAP[DevBAD_COMMANDS.status] = DevBAD_COMMANDS
EXCEPTION_PREFIX_MAP[DevBAD_COMMANDS.prefix] = DevBAD_COMMANDS

class DevINV_SETUP(DevException):
    status = 662470906
    prefix = "%DEV-E-INV_SETUP"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-INV_SETUP, Device was not properly set up")

EXCEPTION_MAP[DevINV_SETUP.status] = DevINV_SETUP
EXCEPTION_PREFIX_MAP[DevINV_SETUP.prefix] = DevINV_SETUP

class DevPYDEVICE_NOT_FOUND(DevException):
    status = 662470914
    prefix = "%DEV-E-PYDEVICE_NOT_FOUND"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-PYDEVICE_NOT_FOUND, Python device class not found")

EXCEPTION_MAP[DevPYDEVICE_NOT_FOUND.status] = DevPYDEVICE_NOT_FOUND
EXCEPTION_PREFIX_MAP[DevPYDEVICE_NOT_FOUND.prefix] = DevPYDEVICE_NOT_FOUND

class DevPY_INTERFACE_LIBRARY_NOT_FOUND(DevException):
    status = 662470922
    prefix = "%DEV-E-PY_INTERFACE_LIBRARY_NOT_FOUND"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-PY_INTERFACE_LIBRARY_NOT_FOUND, The needed device hardware interface library could not be loaded")

EXCEPTION_MAP[DevPY_INTERFACE_LIBRARY_NOT_FOUND.status] = DevPY_INTERFACE_LIBRARY_NOT_FOUND
EXCEPTION_PREFIX_MAP[DevPY_INTERFACE_LIBRARY_NOT_FOUND.prefix] = DevPY_INTERFACE_LIBRARY_NOT_FOUND

class DevIO_STUCK(DevException):
    status = 662470930
    prefix = "%DEV-E-IO_STUCK"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-IO_STUCK, I/O to Device is stuck. Check network connection and board status")

EXCEPTION_MAP[DevIO_STUCK.status] = DevIO_STUCK
EXCEPTION_PREFIX_MAP[DevIO_STUCK.prefix] = DevIO_STUCK

class DevUNKNOWN_STATE(DevException):
    status = 662470938
    prefix = "%DEV-E-UNKNOWN_STATE"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-UNKNOWN_STATE, Device returned unrecognized state string")

EXCEPTION_MAP[DevUNKNOWN_STATE.status] = DevUNKNOWN_STATE
EXCEPTION_PREFIX_MAP[DevUNKNOWN_STATE.prefix] = DevUNKNOWN_STATE

class DevWRONG_TREE(DevException):
    status = 662470946
    prefix = "%DEV-E-WRONG_TREE"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-WRONG_TREE, Attempt to digitizerinto different tree than it was armed with")

EXCEPTION_MAP[DevWRONG_TREE.status] = DevWRONG_TREE
EXCEPTION_PREFIX_MAP[DevWRONG_TREE.prefix] = DevWRONG_TREE

class DevWRONG_PATH(DevException):
    status = 662470954
    prefix = "%DEV-E-WRONG_PATH"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-WRONG_PATH, Attempt to store digitizer into different path than it was armed with")

EXCEPTION_MAP[DevWRONG_PATH.status] = DevWRONG_PATH
EXCEPTION_PREFIX_MAP[DevWRONG_PATH.prefix] = DevWRONG_PATH

class DevWRONG_SHOT(DevException):
    status = 662470962
    prefix = "%DEV-E-WRONG_SHOT"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-WRONG_SHOT, Attempt to store digitizer into different shot than it was armed with")

EXCEPTION_MAP[DevWRONG_SHOT.status] = DevWRONG_SHOT
EXCEPTION_PREFIX_MAP[DevWRONG_SHOT.prefix] = DevWRONG_SHOT

class DevOFFLINE(DevException):
    status = 662470970
    prefix = "%DEV-E-OFFLINE"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-OFFLINE, Device is not on line. Check network connection")

EXCEPTION_MAP[DevOFFLINE.status] = DevOFFLINE
EXCEPTION_PREFIX_MAP[DevOFFLINE.prefix] = DevOFFLINE

class DevTRIGGERED_NOT_STORED(DevException):
    status = 662470978
    prefix = "%DEV-E-TRIGGERED_NOT_STORED"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-TRIGGERED_NOT_STORED, Device was triggered but not stored")

EXCEPTION_MAP[DevTRIGGERED_NOT_STORED.status] = DevTRIGGERED_NOT_STORED
EXCEPTION_PREFIX_MAP[DevTRIGGERED_NOT_STORED.prefix] = DevTRIGGERED_NOT_STORED

class DevNO_NAME_SPECIFIED(DevException):
    status = 662470986
    prefix = "%DEV-E-NO_NAME_SPECIFIED"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-NO_NAME_SPECIFIED, Device name must be specifed - pleas fill it in")

EXCEPTION_MAP[DevNO_NAME_SPECIFIED.status] = DevNO_NAME_SPECIFIED
EXCEPTION_PREFIX_MAP[DevNO_NAME_SPECIFIED.prefix] = DevNO_NAME_SPECIFIED

class DevBAD_ACTIVE_CHAN(DevException):
    status = 662470994
    prefix = "%DEV-E-BAD_ACTIVE_CHAN"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_ACTIVE_CHAN, Active channels either not available or invalid")

EXCEPTION_MAP[DevBAD_ACTIVE_CHAN.status] = DevBAD_ACTIVE_CHAN
EXCEPTION_PREFIX_MAP[DevBAD_ACTIVE_CHAN.prefix] = DevBAD_ACTIVE_CHAN

class DevBAD_TRIG_SRC(DevException):
    status = 662471002
    prefix = "%DEV-E-BAD_TRIG_SRC"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_TRIG_SRC, Trigger source either not available or invalid")

EXCEPTION_MAP[DevBAD_TRIG_SRC.status] = DevBAD_TRIG_SRC
EXCEPTION_PREFIX_MAP[DevBAD_TRIG_SRC.prefix] = DevBAD_TRIG_SRC

class DevBAD_CLOCK_SRC(DevException):
    status = 662471010
    prefix = "%DEV-E-BAD_CLOCK_SRC"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_CLOCK_SRC, Clock source either not available or invalid")

EXCEPTION_MAP[DevBAD_CLOCK_SRC.status] = DevBAD_CLOCK_SRC
EXCEPTION_PREFIX_MAP[DevBAD_CLOCK_SRC.prefix] = DevBAD_CLOCK_SRC

class DevBAD_PRE_TRIG(DevException):
    status = 662471018
    prefix = "%DEV-E-BAD_PRE_TRIG"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_PRE_TRIG, Pre trigger samples either not available or invalid")

EXCEPTION_MAP[DevBAD_PRE_TRIG.status] = DevBAD_PRE_TRIG
EXCEPTION_PREFIX_MAP[DevBAD_PRE_TRIG.prefix] = DevBAD_PRE_TRIG

class DevBAD_POST_TRIG(DevException):
    status = 662471026
    prefix = "%DEV-E-BAD_POST_TRIG"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_POST_TRIG, Post trigger samples either not available or invalid")

EXCEPTION_MAP[DevBAD_POST_TRIG.status] = DevBAD_POST_TRIG
EXCEPTION_PREFIX_MAP[DevBAD_POST_TRIG.prefix] = DevBAD_POST_TRIG

class DevBAD_CLOCK_FREQ(DevException):
    status = 662471034
    prefix = "%DEV-E-BAD_CLOCK_FREQ"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_CLOCK_FREQ, Clock frequency either not available or invalid")

EXCEPTION_MAP[DevBAD_CLOCK_FREQ.status] = DevBAD_CLOCK_FREQ
EXCEPTION_PREFIX_MAP[DevBAD_CLOCK_FREQ.prefix] = DevBAD_CLOCK_FREQ

class DevTRIGGER_FAILED(DevException):
    status = 662471042
    prefix = "%DEV-E-TRIGGER_FAILED"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-TRIGGER_FAILED, Device trigger method failed")

EXCEPTION_MAP[DevTRIGGER_FAILED.status] = DevTRIGGER_FAILED
EXCEPTION_PREFIX_MAP[DevTRIGGER_FAILED.prefix] = DevTRIGGER_FAILED

class DevERROR_READING_CHANNEL(DevException):
    status = 662471050
    prefix = "%DEV-E-ERROR_READING_CHANNEL"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-ERROR_READING_CHANNEL, Error reading data for channel from device")

EXCEPTION_MAP[DevERROR_READING_CHANNEL.status] = DevERROR_READING_CHANNEL
EXCEPTION_PREFIX_MAP[DevERROR_READING_CHANNEL.prefix] = DevERROR_READING_CHANNEL

class DevERROR_DOING_INIT(DevException):
    status = 662471058
    prefix = "%DEV-E-ERROR_DOING_INIT"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-ERROR_DOING_INIT, Error sending ARM command to device")

EXCEPTION_MAP[DevERROR_DOING_INIT.status] = DevERROR_DOING_INIT
EXCEPTION_PREFIX_MAP[DevERROR_DOING_INIT.prefix] = DevERROR_DOING_INIT

class DevCANNOT_LOAD_SETTINGS(DevException):
    status = 662480290
    prefix = "%DEV-E-CANNOT_LOAD_SETTINGS"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-CANNOT_LOAD_SETTINGS, Error loading settings from XML or JSON")

EXCEPTION_MAP[DevCANNOT_LOAD_SETTINGS.status] = DevCANNOT_LOAD_SETTINGS
EXCEPTION_PREFIX_MAP[DevCANNOT_LOAD_SETTINGS.prefix] = DevCANNOT_LOAD_SETTINGS

class DevCANNOT_GET_BOARD_STATE(DevException):
    status = 662480298
    prefix = "%DEV-E-CANNOT_GET_BOARD_STATE"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-CANNOT_GET_BOARD_STATE, Cannot retrieve state of daq board")

EXCEPTION_MAP[DevCANNOT_GET_BOARD_STATE.status] = DevCANNOT_GET_BOARD_STATE
EXCEPTION_PREFIX_MAP[DevCANNOT_GET_BOARD_STATE.prefix] = DevCANNOT_GET_BOARD_STATE

class DevACQCMD_FAILED(DevException):
    status = 662480306
    prefix = "%DEV-E-ACQCMD_FAILED"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-ACQCMD_FAILED, Error executing acqcmd on daq board")

EXCEPTION_MAP[DevACQCMD_FAILED.status] = DevACQCMD_FAILED
EXCEPTION_PREFIX_MAP[DevACQCMD_FAILED.prefix] = DevACQCMD_FAILED

class DevACQ2SH_FAILED(DevException):
    status = 662480314
    prefix = "%DEV-E-ACQ2SH_FAILED"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-ACQ2SH_FAILED, Error executing acq2sh command on daq board")

EXCEPTION_MAP[DevACQ2SH_FAILED.status] = DevACQ2SH_FAILED
EXCEPTION_PREFIX_MAP[DevACQ2SH_FAILED.prefix] = DevACQ2SH_FAILED

class DevBAD_PARAMETER(DevException):
    status = 662480322
    prefix = "%DEV-E-BAD_PARAMETER"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-BAD_PARAMETER, Invalid parameter specified for device")

EXCEPTION_MAP[DevBAD_PARAMETER.status] = DevBAD_PARAMETER
EXCEPTION_PREFIX_MAP[DevBAD_PARAMETER.prefix] = DevBAD_PARAMETER

class DevCOMM_ERROR(DevException):
    status = 662480330
    prefix = "%DEV-E-COMM_ERROR"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-COMM_ERROR, Error communicating with device")

EXCEPTION_MAP[DevCOMM_ERROR.status] = DevCOMM_ERROR
EXCEPTION_PREFIX_MAP[DevCOMM_ERROR.prefix] = DevCOMM_ERROR

class DevCAMERA_NOT_FOUND(DevException):
    status = 662480338
    prefix = "%DEV-E-CAMERA_NOT_FOUND"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-CAMERA_NOT_FOUND, Could not find specified camera on the network")

EXCEPTION_MAP[DevCAMERA_NOT_FOUND.status] = DevCAMERA_NOT_FOUND
EXCEPTION_PREFIX_MAP[DevCAMERA_NOT_FOUND.prefix] = DevCAMERA_NOT_FOUND

class DevNOT_A_PYDEVICE(DevException):
    status = 662480346
    prefix = "%DEV-E-NOT_A_PYDEVICE"
    def __init__(self):
        Exception.__init__(self, "%DEV-E-NOT_A_PYDEVICE, Device is not a python device")

EXCEPTION_MAP[DevNOT_A_PYDEVICE.status] = DevNOT_A_PYDEVICE
EXCEPTION_PREFIX_MAP[DevNOT_A_PYDEVICE.prefix] = DevNOT_A_PYDEVICE

class TreeException(MdsException):
    pass

class TreeALREADY_OFF(TreeException):
    status = 265388075
    prefix = "%TREE-I-ALREADY_OFF"
    def __init__(self):
        Exception.__init__(self, "%TREE-I-ALREADY_OFF, Node is already OFF")

EXCEPTION_MAP[TreeALREADY_OFF.status] = TreeALREADY_OFF
EXCEPTION_PREFIX_MAP[TreeALREADY_OFF.prefix] = TreeALREADY_OFF

class TreeALREADY_ON(TreeException):
    status = 265388083
    prefix = "%TREE-I-ALREADY_ON"
    def __init__(self):
        Exception.__init__(self, "%TREE-I-ALREADY_ON, Node is already ON")

EXCEPTION_MAP[TreeALREADY_ON.status] = TreeALREADY_ON
EXCEPTION_PREFIX_MAP[TreeALREADY_ON.prefix] = TreeALREADY_ON

class TreeALREADY_OPEN(TreeException):
    status = 265388091
    prefix = "%TREE-I-ALREADY_OPEN"
    def __init__(self):
        Exception.__init__(self, "%TREE-I-ALREADY_OPEN, Tree is already OPEN")

EXCEPTION_MAP[TreeALREADY_OPEN.status] = TreeALREADY_OPEN
EXCEPTION_PREFIX_MAP[TreeALREADY_OPEN.prefix] = TreeALREADY_OPEN

class TreeALREADY_THERE(TreeException):
    status = 265388168
    prefix = "%TREE-W-ALREADY_THERE"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-ALREADY_THERE, Node is already in the tree")

EXCEPTION_MAP[TreeALREADY_THERE.status] = TreeALREADY_THERE
EXCEPTION_PREFIX_MAP[TreeALREADY_THERE.prefix] = TreeALREADY_THERE

class TreeBADRECORD(TreeException):
    status = 265388218
    prefix = "%TREE-E-BADRECORD"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-BADRECORD, Data corrupted: cannot read record")

EXCEPTION_MAP[TreeBADRECORD.status] = TreeBADRECORD
EXCEPTION_PREFIX_MAP[TreeBADRECORD.prefix] = TreeBADRECORD

class TreeBOTH_OFF(TreeException):
    status = 265388184
    prefix = "%TREE-W-BOTH_OFF"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-BOTH_OFF, Both this node and its parent are off")

EXCEPTION_MAP[TreeBOTH_OFF.status] = TreeBOTH_OFF
EXCEPTION_PREFIX_MAP[TreeBOTH_OFF.prefix] = TreeBOTH_OFF

class TreeBUFFEROVF(TreeException):
    status = 265388306
    prefix = "%TREE-E-BUFFEROVF"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-BUFFEROVF, Output buffer overflow")

EXCEPTION_MAP[TreeBUFFEROVF.status] = TreeBUFFEROVF
EXCEPTION_PREFIX_MAP[TreeBUFFEROVF.prefix] = TreeBUFFEROVF

class TreeCONGLOMFULL(TreeException):
    status = 265388322
    prefix = "%TREE-E-CONGLOMFULL"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-CONGLOMFULL, Current conglomerate is full")

EXCEPTION_MAP[TreeCONGLOMFULL.status] = TreeCONGLOMFULL
EXCEPTION_PREFIX_MAP[TreeCONGLOMFULL.prefix] = TreeCONGLOMFULL

class TreeCONGLOM_NOT_FULL(TreeException):
    status = 265388330
    prefix = "%TREE-E-CONGLOM_NOT_FULL"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-CONGLOM_NOT_FULL, Current conglomerate is not yet full")

EXCEPTION_MAP[TreeCONGLOM_NOT_FULL.status] = TreeCONGLOM_NOT_FULL
EXCEPTION_PREFIX_MAP[TreeCONGLOM_NOT_FULL.prefix] = TreeCONGLOM_NOT_FULL

class TreeCONTINUING(TreeException):
    status = 265390435
    prefix = "%TREE-I-CONTINUING"
    def __init__(self):
        Exception.__init__(self, "%TREE-I-CONTINUING, Operation continuing: note following error")

EXCEPTION_MAP[TreeCONTINUING.status] = TreeCONTINUING
EXCEPTION_PREFIX_MAP[TreeCONTINUING.prefix] = TreeCONTINUING

class TreeDUPTAG(TreeException):
    status = 265388234
    prefix = "%TREE-E-DUPTAG"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-DUPTAG, Tag name already in use")

EXCEPTION_MAP[TreeDUPTAG.status] = TreeDUPTAG
EXCEPTION_PREFIX_MAP[TreeDUPTAG.prefix] = TreeDUPTAG

class TreeEDITING(TreeException):
    status = 265388434
    prefix = "%TREE-E-EDITING"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-EDITING, Tree file open for edit: operation not permitted")

EXCEPTION_MAP[TreeEDITING.status] = TreeEDITING
EXCEPTION_PREFIX_MAP[TreeEDITING.prefix] = TreeEDITING

class TreeILLEGAL_ITEM(TreeException):
    status = 265388298
    prefix = "%TREE-E-ILLEGAL_ITEM"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-ILLEGAL_ITEM, Invalid item code or part number specified")

EXCEPTION_MAP[TreeILLEGAL_ITEM.status] = TreeILLEGAL_ITEM
EXCEPTION_PREFIX_MAP[TreeILLEGAL_ITEM.prefix] = TreeILLEGAL_ITEM

class TreeILLPAGCNT(TreeException):
    status = 265388242
    prefix = "%TREE-E-ILLPAGCNT"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-ILLPAGCNT, Illegal page count, error mapping tree file")

EXCEPTION_MAP[TreeILLPAGCNT.status] = TreeILLPAGCNT
EXCEPTION_PREFIX_MAP[TreeILLPAGCNT.prefix] = TreeILLPAGCNT

class TreeINVDFFCLASS(TreeException):
    status = 265388346
    prefix = "%TREE-E-INVDFFCLASS"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-INVDFFCLASS, Invalid data fmt: only CLASS_S can have data in NCI")

EXCEPTION_MAP[TreeINVDFFCLASS.status] = TreeINVDFFCLASS
EXCEPTION_PREFIX_MAP[TreeINVDFFCLASS.prefix] = TreeINVDFFCLASS

class TreeINVDTPUSG(TreeException):
    status = 265388426
    prefix = "%TREE-E-INVDTPUSG"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-INVDTPUSG, Attempt to store datatype which conflicts with the designated usage of this node")

EXCEPTION_MAP[TreeINVDTPUSG.status] = TreeINVDTPUSG
EXCEPTION_PREFIX_MAP[TreeINVDTPUSG.prefix] = TreeINVDTPUSG

class TreeINVPATH(TreeException):
    status = 265388290
    prefix = "%TREE-E-INVPATH"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-INVPATH, Invalid tree pathname specified")

EXCEPTION_MAP[TreeINVPATH.status] = TreeINVPATH
EXCEPTION_PREFIX_MAP[TreeINVPATH.prefix] = TreeINVPATH

class TreeINVRECTYP(TreeException):
    status = 265388354
    prefix = "%TREE-E-INVRECTYP"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-INVRECTYP, Record type invalid for requested operation")

EXCEPTION_MAP[TreeINVRECTYP.status] = TreeINVRECTYP
EXCEPTION_PREFIX_MAP[TreeINVRECTYP.prefix] = TreeINVRECTYP

class TreeINVTREE(TreeException):
    status = 265388226
    prefix = "%TREE-E-INVTREE"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-INVTREE, Invalid tree identification structure")

EXCEPTION_MAP[TreeINVTREE.status] = TreeINVTREE
EXCEPTION_PREFIX_MAP[TreeINVTREE.prefix] = TreeINVTREE

class TreeMAXOPENEDIT(TreeException):
    status = 265388250
    prefix = "%TREE-E-MAXOPENEDIT"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-MAXOPENEDIT, Too many files open for edit")

EXCEPTION_MAP[TreeMAXOPENEDIT.status] = TreeMAXOPENEDIT
EXCEPTION_PREFIX_MAP[TreeMAXOPENEDIT.prefix] = TreeMAXOPENEDIT

class TreeNEW(TreeException):
    status = 265388059
    prefix = "%TREE-I-NEW"
    def __init__(self):
        Exception.__init__(self, "%TREE-I-NEW, New tree created")

EXCEPTION_MAP[TreeNEW.status] = TreeNEW
EXCEPTION_PREFIX_MAP[TreeNEW.prefix] = TreeNEW

class TreeNMN(TreeException):
    status = 265388128
    prefix = "%TREE-W-NMN"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-NMN, No More Nodes")

EXCEPTION_MAP[TreeNMN.status] = TreeNMN
EXCEPTION_PREFIX_MAP[TreeNMN.prefix] = TreeNMN

class TreeNMT(TreeException):
    status = 265388136
    prefix = "%TREE-W-NMT"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-NMT, No More Tags")

EXCEPTION_MAP[TreeNMT.status] = TreeNMT
EXCEPTION_PREFIX_MAP[TreeNMT.prefix] = TreeNMT

class TreeNNF(TreeException):
    status = 265388144
    prefix = "%TREE-W-NNF"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-NNF, Node Not Found")

EXCEPTION_MAP[TreeNNF.status] = TreeNNF
EXCEPTION_PREFIX_MAP[TreeNNF.prefix] = TreeNNF

class TreeNODATA(TreeException):
    status = 265388258
    prefix = "%TREE-E-NODATA"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NODATA, No data available for this node")

EXCEPTION_MAP[TreeNODATA.status] = TreeNODATA
EXCEPTION_PREFIX_MAP[TreeNODATA.prefix] = TreeNODATA

class TreeNODNAMLEN(TreeException):
    status = 265388362
    prefix = "%TREE-E-NODNAMLEN"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NODNAMLEN, Node name too long (12 chars max)")

EXCEPTION_MAP[TreeNODNAMLEN.status] = TreeNODNAMLEN
EXCEPTION_PREFIX_MAP[TreeNODNAMLEN.prefix] = TreeNODNAMLEN

class TreeNOEDIT(TreeException):
    status = 265388274
    prefix = "%TREE-E-NOEDIT"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOEDIT, Tree file is not open for edit")

EXCEPTION_MAP[TreeNOEDIT.status] = TreeNOEDIT
EXCEPTION_PREFIX_MAP[TreeNOEDIT.prefix] = TreeNOEDIT

class TreeNOLOG(TreeException):
    status = 265388458
    prefix = "%TREE-E-NOLOG"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOLOG, Experiment pathname (xxx_path) not defined")

EXCEPTION_MAP[TreeNOLOG.status] = TreeNOLOG
EXCEPTION_PREFIX_MAP[TreeNOLOG.prefix] = TreeNOLOG

class TreeNOMETHOD(TreeException):
    status = 265388208
    prefix = "%TREE-W-NOMETHOD"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-NOMETHOD, Method not available for this object")

EXCEPTION_MAP[TreeNOMETHOD.status] = TreeNOMETHOD
EXCEPTION_PREFIX_MAP[TreeNOMETHOD.prefix] = TreeNOMETHOD

class TreeNOOVERWRITE(TreeException):
    status = 265388418
    prefix = "%TREE-E-NOOVERWRITE"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOOVERWRITE, Write-once node: overwrite not permitted")

EXCEPTION_MAP[TreeNOOVERWRITE.status] = TreeNOOVERWRITE
EXCEPTION_PREFIX_MAP[TreeNOOVERWRITE.prefix] = TreeNOOVERWRITE

class TreeNOTALLSUBS(TreeException):
    status = 265388067
    prefix = "%TREE-I-NOTALLSUBS"
    def __init__(self):
        Exception.__init__(self, "%TREE-I-NOTALLSUBS, Main tree opened but not all subtrees found/or connected")

EXCEPTION_MAP[TreeNOTALLSUBS.status] = TreeNOTALLSUBS
EXCEPTION_PREFIX_MAP[TreeNOTALLSUBS.prefix] = TreeNOTALLSUBS

class TreeNOTCHILDLESS(TreeException):
    status = 265388282
    prefix = "%TREE-E-NOTCHILDLESS"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOTCHILDLESS, Node must be childless to become subtree reference")

EXCEPTION_MAP[TreeNOTCHILDLESS.status] = TreeNOTCHILDLESS
EXCEPTION_PREFIX_MAP[TreeNOTCHILDLESS.prefix] = TreeNOTCHILDLESS

class TreeNOT_IN_LIST(TreeException):
    status = 265388482
    prefix = "%TREE-E-NOT_IN_LIST"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOT_IN_LIST, Tree being opened was not in the list")

EXCEPTION_MAP[TreeNOT_IN_LIST.status] = TreeNOT_IN_LIST
EXCEPTION_PREFIX_MAP[TreeNOT_IN_LIST.prefix] = TreeNOT_IN_LIST

class TreeNOTMEMBERLESS(TreeException):
    status = 265388402
    prefix = "%TREE-E-NOTMEMBERLESS"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOTMEMBERLESS, Subtree reference can not have members")

EXCEPTION_MAP[TreeNOTMEMBERLESS.status] = TreeNOTMEMBERLESS
EXCEPTION_PREFIX_MAP[TreeNOTMEMBERLESS.prefix] = TreeNOTMEMBERLESS

class TreeNOTSON(TreeException):
    status = 265388410
    prefix = "%TREE-E-NOTSON"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOTSON, Subtree reference cannot be a member")

EXCEPTION_MAP[TreeNOTSON.status] = TreeNOTSON
EXCEPTION_PREFIX_MAP[TreeNOTSON.prefix] = TreeNOTSON

class TreeNOT_CONGLOM(TreeException):
    status = 265388386
    prefix = "%TREE-E-NOT_CONGLOM"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOT_CONGLOM, Head node of conglomerate does not contain a DTYPE_CONGLOM record")

EXCEPTION_MAP[TreeNOT_CONGLOM.status] = TreeNOT_CONGLOM
EXCEPTION_PREFIX_MAP[TreeNOT_CONGLOM.prefix] = TreeNOT_CONGLOM

class TreeNOT_OPEN(TreeException):
    status = 265388200
    prefix = "%TREE-W-NOT_OPEN"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-NOT_OPEN, Tree not currently open")

EXCEPTION_MAP[TreeNOT_OPEN.status] = TreeNOT_OPEN
EXCEPTION_PREFIX_MAP[TreeNOT_OPEN.prefix] = TreeNOT_OPEN

class TreeNOWRITEMODEL(TreeException):
    status = 265388442
    prefix = "%TREE-E-NOWRITEMODEL"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOWRITEMODEL, Data for this node can not be written into the MODEL file")

EXCEPTION_MAP[TreeNOWRITEMODEL.status] = TreeNOWRITEMODEL
EXCEPTION_PREFIX_MAP[TreeNOWRITEMODEL.prefix] = TreeNOWRITEMODEL

class TreeNOWRITESHOT(TreeException):
    status = 265388450
    prefix = "%TREE-E-NOWRITESHOT"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOWRITESHOT, Data for this node can not be written into the SHOT file")

EXCEPTION_MAP[TreeNOWRITESHOT.status] = TreeNOWRITESHOT
EXCEPTION_PREFIX_MAP[TreeNOWRITESHOT.prefix] = TreeNOWRITESHOT

class TreeNO_CONTEXT(TreeException):
    status = 265388099
    prefix = "%TREE-I-NO_CONTEXT"
    def __init__(self):
        Exception.__init__(self, "%TREE-I-NO_CONTEXT, There is no active search to end")

EXCEPTION_MAP[TreeNO_CONTEXT.status] = TreeNO_CONTEXT
EXCEPTION_PREFIX_MAP[TreeNO_CONTEXT.prefix] = TreeNO_CONTEXT

class TreeOFF(TreeException):
    status = 265388192
    prefix = "%TREE-W-OFF"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-OFF, Node is OFF")

EXCEPTION_MAP[TreeOFF.status] = TreeOFF
EXCEPTION_PREFIX_MAP[TreeOFF.prefix] = TreeOFF

class TreeON(TreeException):
    status = 265388107
    prefix = "%TREE-I-ON"
    def __init__(self):
        Exception.__init__(self, "%TREE-I-ON, Node is ON")

EXCEPTION_MAP[TreeON.status] = TreeON
EXCEPTION_PREFIX_MAP[TreeON.prefix] = TreeON

class TreeOPEN(TreeException):
    status = 265388115
    prefix = "%TREE-I-OPEN"
    def __init__(self):
        Exception.__init__(self, "%TREE-I-OPEN, Tree is OPEN (no edit)")

EXCEPTION_MAP[TreeOPEN.status] = TreeOPEN
EXCEPTION_PREFIX_MAP[TreeOPEN.prefix] = TreeOPEN

class TreeOPEN_EDIT(TreeException):
    status = 265388123
    prefix = "%TREE-I-OPEN_EDIT"
    def __init__(self):
        Exception.__init__(self, "%TREE-I-OPEN_EDIT, Tree is OPEN for edit")

EXCEPTION_MAP[TreeOPEN_EDIT.status] = TreeOPEN_EDIT
EXCEPTION_PREFIX_MAP[TreeOPEN_EDIT.prefix] = TreeOPEN_EDIT

class TreePARENT_OFF(TreeException):
    status = 265388176
    prefix = "%TREE-W-PARENT_OFF"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-PARENT_OFF, Parent of this node is OFF")

EXCEPTION_MAP[TreePARENT_OFF.status] = TreePARENT_OFF
EXCEPTION_PREFIX_MAP[TreePARENT_OFF.prefix] = TreePARENT_OFF

class TreeREADERR(TreeException):
    status = 265388474
    prefix = "%TREE-E-READERR"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-READERR, Error reading record for node")

EXCEPTION_MAP[TreeREADERR.status] = TreeREADERR
EXCEPTION_PREFIX_MAP[TreeREADERR.prefix] = TreeREADERR

class TreeREADONLY(TreeException):
    status = 265388466
    prefix = "%TREE-E-READONLY"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-READONLY, Tree was opened with readonly access")

EXCEPTION_MAP[TreeREADONLY.status] = TreeREADONLY
EXCEPTION_PREFIX_MAP[TreeREADONLY.prefix] = TreeREADONLY

class TreeRESOLVED(TreeException):
    status = 265388049
    prefix = "%TREE-S-RESOLVED"
    def __init__(self):
        Exception.__init__(self, "%TREE-S-RESOLVED, Indirect reference successfully resolved")

EXCEPTION_MAP[TreeRESOLVED.status] = TreeRESOLVED
EXCEPTION_PREFIX_MAP[TreeRESOLVED.prefix] = TreeRESOLVED

class TreeSUCCESS(TreeException):
    status = 265389633
    prefix = "%TREE-S-SUCCESS"
    def __init__(self):
        Exception.__init__(self, "%TREE-S-SUCCESS, Operation successful")

EXCEPTION_MAP[TreeSUCCESS.status] = TreeSUCCESS
EXCEPTION_PREFIX_MAP[TreeSUCCESS.prefix] = TreeSUCCESS

class TreeTAGNAMLEN(TreeException):
    status = 265388370
    prefix = "%TREE-E-TAGNAMLEN"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-TAGNAMLEN, Tagname too long (max 24 chars)")

EXCEPTION_MAP[TreeTAGNAMLEN.status] = TreeTAGNAMLEN
EXCEPTION_PREFIX_MAP[TreeTAGNAMLEN.prefix] = TreeTAGNAMLEN

class TreeTNF(TreeException):
    status = 265388152
    prefix = "%TREE-W-TNF"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-TNF, Tag Not Found")

EXCEPTION_MAP[TreeTNF.status] = TreeTNF
EXCEPTION_PREFIX_MAP[TreeTNF.prefix] = TreeTNF

class TreeTREENF(TreeException):
    status = 265388160
    prefix = "%TREE-W-TREENF"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-TREENF, Tree Not Found")

EXCEPTION_MAP[TreeTREENF.status] = TreeTREENF
EXCEPTION_PREFIX_MAP[TreeTREENF.prefix] = TreeTREENF

class TreeUNRESOLVED(TreeException):
    status = 265388338
    prefix = "%TREE-E-UNRESOLVED"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-UNRESOLVED, Not an indirect node reference: No action taken")

EXCEPTION_MAP[TreeUNRESOLVED.status] = TreeUNRESOLVED
EXCEPTION_PREFIX_MAP[TreeUNRESOLVED.prefix] = TreeUNRESOLVED

class TreeUNSPRTCLASS(TreeException):
    status = 265388314
    prefix = "%TREE-E-UNSPRTCLASS"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-UNSPRTCLASS, Unsupported descriptor class")

EXCEPTION_MAP[TreeUNSPRTCLASS.status] = TreeUNSPRTCLASS
EXCEPTION_PREFIX_MAP[TreeUNSPRTCLASS.prefix] = TreeUNSPRTCLASS

class TreeUNSUPARRDTYPE(TreeException):
    status = 265388394
    prefix = "%TREE-E-UNSUPARRDTYPE"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-UNSUPARRDTYPE, Complex data types not supported as members of arrays")

EXCEPTION_MAP[TreeUNSUPARRDTYPE.status] = TreeUNSUPARRDTYPE
EXCEPTION_PREFIX_MAP[TreeUNSUPARRDTYPE.prefix] = TreeUNSUPARRDTYPE

class TreeWRITEFIRST(TreeException):
    status = 265388378
    prefix = "%TREE-E-WRITEFIRST"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-WRITEFIRST, Tree has been modified: write or quit first")

EXCEPTION_MAP[TreeWRITEFIRST.status] = TreeWRITEFIRST
EXCEPTION_PREFIX_MAP[TreeWRITEFIRST.prefix] = TreeWRITEFIRST

class TreeFAILURE(TreeException):
    status = 265392034
    prefix = "%TREE-E-FAILURE"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-FAILURE, Operation NOT successful")

EXCEPTION_MAP[TreeFAILURE.status] = TreeFAILURE
EXCEPTION_PREFIX_MAP[TreeFAILURE.prefix] = TreeFAILURE

class TreeLOCK_FAILURE(TreeException):
    status = 265392048
    prefix = "%TREE-W-LOCK_FAILURE"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-LOCK_FAILURE, Error locking file, perhaps NFSLOCKING not enabled on this system")

EXCEPTION_MAP[TreeLOCK_FAILURE.status] = TreeLOCK_FAILURE
EXCEPTION_PREFIX_MAP[TreeLOCK_FAILURE.prefix] = TreeLOCK_FAILURE

class TreeFILE_NOT_FOUND(TreeException):
    status = 265392042
    prefix = "%TREE-E-FILE_NOT_FOUND"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-FILE_NOT_FOUND, File or Directory Not Found")

EXCEPTION_MAP[TreeFILE_NOT_FOUND.status] = TreeFILE_NOT_FOUND
EXCEPTION_PREFIX_MAP[TreeFILE_NOT_FOUND.prefix] = TreeFILE_NOT_FOUND

class TreeCANCEL(TreeException):
    status = 265391232
    prefix = "%TREE-W-CANCEL"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-CANCEL, User canceled operation")

EXCEPTION_MAP[TreeCANCEL.status] = TreeCANCEL
EXCEPTION_PREFIX_MAP[TreeCANCEL.prefix] = TreeCANCEL

class TreeUNSUPTHICKOP(TreeException):
    status = 265391240
    prefix = "%TREE-W-UNSUPTHICKOP"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-UNSUPTHICKOP, Unsupported thick client operation")

EXCEPTION_MAP[TreeUNSUPTHICKOP.status] = TreeUNSUPTHICKOP
EXCEPTION_PREFIX_MAP[TreeUNSUPTHICKOP.prefix] = TreeUNSUPTHICKOP

class TreeNOSEGMENTS(TreeException):
    status = 265392058
    prefix = "%TREE-E-NOSEGMENTS"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOSEGMENTS, No segments exist in this node")

EXCEPTION_MAP[TreeNOSEGMENTS.status] = TreeNOSEGMENTS
EXCEPTION_PREFIX_MAP[TreeNOSEGMENTS.prefix] = TreeNOSEGMENTS

class TreeINVDTYPE(TreeException):
    status = 265392066
    prefix = "%TREE-E-INVDTYPE"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-INVDTYPE, Invalid datatype for data segment")

EXCEPTION_MAP[TreeINVDTYPE.status] = TreeINVDTYPE
EXCEPTION_PREFIX_MAP[TreeINVDTYPE.prefix] = TreeINVDTYPE

class TreeINVSHAPE(TreeException):
    status = 265392074
    prefix = "%TREE-E-INVSHAPE"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-INVSHAPE, Invalid shape for this data segment")

EXCEPTION_MAP[TreeINVSHAPE.status] = TreeINVSHAPE
EXCEPTION_PREFIX_MAP[TreeINVSHAPE.prefix] = TreeINVSHAPE

class TreeINVSHOT(TreeException):
    status = 265392090
    prefix = "%TREE-E-INVSHOT"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-INVSHOT, Invalid shot number - must be -1 (model), 0 (current), or Positive")

EXCEPTION_MAP[TreeINVSHOT.status] = TreeINVSHOT
EXCEPTION_PREFIX_MAP[TreeINVSHOT.prefix] = TreeINVSHOT

class TreeINVTAG(TreeException):
    status = 265392106
    prefix = "%TREE-E-INVTAG"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-INVTAG, Invalid tagname - must begin with alpha followed by 0-22 alphanumeric or underscores")

EXCEPTION_MAP[TreeINVTAG.status] = TreeINVTAG
EXCEPTION_PREFIX_MAP[TreeINVTAG.prefix] = TreeINVTAG

class TreeNOPATH(TreeException):
    status = 265392114
    prefix = "%TREE-E-NOPATH"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOPATH, No 'treename'_path or default_tree_path environment variables defined. Cannot locate tree files")

EXCEPTION_MAP[TreeNOPATH.status] = TreeNOPATH
EXCEPTION_PREFIX_MAP[TreeNOPATH.prefix] = TreeNOPATH

class TreeTREEFILEREADERR(TreeException):
    status = 265392122
    prefix = "%TREE-E-TREEFILEREADERR"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-TREEFILEREADERR, Error reading in tree file contents")

EXCEPTION_MAP[TreeTREEFILEREADERR.status] = TreeTREEFILEREADERR
EXCEPTION_PREFIX_MAP[TreeTREEFILEREADERR.prefix] = TreeTREEFILEREADERR

class TreeMEMERR(TreeException):
    status = 265392130
    prefix = "%TREE-E-MEMERR"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-MEMERR, Memory allocation error")

EXCEPTION_MAP[TreeMEMERR.status] = TreeMEMERR
EXCEPTION_PREFIX_MAP[TreeMEMERR.prefix] = TreeMEMERR

class TreeNOCURRENT(TreeException):
    status = 265392138
    prefix = "%TREE-E-NOCURRENT"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOCURRENT, No current shot number set for this tree")

EXCEPTION_MAP[TreeNOCURRENT.status] = TreeNOCURRENT
EXCEPTION_PREFIX_MAP[TreeNOCURRENT.prefix] = TreeNOCURRENT

class TreeFOPENW(TreeException):
    status = 265392146
    prefix = "%TREE-E-FOPENW"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-FOPENW, Error opening file for read-write")

EXCEPTION_MAP[TreeFOPENW.status] = TreeFOPENW
EXCEPTION_PREFIX_MAP[TreeFOPENW.prefix] = TreeFOPENW

class TreeFOPENR(TreeException):
    status = 265392154
    prefix = "%TREE-E-FOPENR"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-FOPENR, Error opening file read-only")

EXCEPTION_MAP[TreeFOPENR.status] = TreeFOPENR
EXCEPTION_PREFIX_MAP[TreeFOPENR.prefix] = TreeFOPENR

class TreeFCREATE(TreeException):
    status = 265392162
    prefix = "%TREE-E-FCREATE"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-FCREATE, Error creating new file")

EXCEPTION_MAP[TreeFCREATE.status] = TreeFCREATE
EXCEPTION_PREFIX_MAP[TreeFCREATE.prefix] = TreeFCREATE

class TreeCONNECTFAIL(TreeException):
    status = 265392170
    prefix = "%TREE-E-CONNECTFAIL"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-CONNECTFAIL, Error connecting to remote server")

EXCEPTION_MAP[TreeCONNECTFAIL.status] = TreeCONNECTFAIL
EXCEPTION_PREFIX_MAP[TreeCONNECTFAIL.prefix] = TreeCONNECTFAIL

class TreeNCIWRITE(TreeException):
    status = 265392178
    prefix = "%TREE-E-NCIWRITE"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NCIWRITE, Error writing node characterisitics to file")

EXCEPTION_MAP[TreeNCIWRITE.status] = TreeNCIWRITE
EXCEPTION_PREFIX_MAP[TreeNCIWRITE.prefix] = TreeNCIWRITE

class TreeDELFAIL(TreeException):
    status = 265392186
    prefix = "%TREE-E-DELFAIL"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-DELFAIL, Error deleting file")

EXCEPTION_MAP[TreeDELFAIL.status] = TreeDELFAIL
EXCEPTION_PREFIX_MAP[TreeDELFAIL.prefix] = TreeDELFAIL

class TreeRENFAIL(TreeException):
    status = 265392194
    prefix = "%TREE-E-RENFAIL"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-RENFAIL, Error renaming file")

EXCEPTION_MAP[TreeRENFAIL.status] = TreeRENFAIL
EXCEPTION_PREFIX_MAP[TreeRENFAIL.prefix] = TreeRENFAIL

class TreeEMPTY(TreeException):
    status = 265392200
    prefix = "%TREE-W-EMPTY"
    def __init__(self):
        Exception.__init__(self, "%TREE-W-EMPTY, Empty string provided")

EXCEPTION_MAP[TreeEMPTY.status] = TreeEMPTY
EXCEPTION_PREFIX_MAP[TreeEMPTY.prefix] = TreeEMPTY

class TreePARSEERR(TreeException):
    status = 265392210
    prefix = "%TREE-E-PARSEERR"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-PARSEERR, Invalid node search string")

EXCEPTION_MAP[TreePARSEERR.status] = TreePARSEERR
EXCEPTION_PREFIX_MAP[TreePARSEERR.prefix] = TreePARSEERR

class TreeNCIREAD(TreeException):
    status = 265392218
    prefix = "%TREE-E-NCIREAD"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NCIREAD, Error reading node characteristics from file")

EXCEPTION_MAP[TreeNCIREAD.status] = TreeNCIREAD
EXCEPTION_PREFIX_MAP[TreeNCIREAD.prefix] = TreeNCIREAD

class TreeNOVERSION(TreeException):
    status = 265392226
    prefix = "%TREE-E-NOVERSION"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOVERSION, No version available")

EXCEPTION_MAP[TreeNOVERSION.status] = TreeNOVERSION
EXCEPTION_PREFIX_MAP[TreeNOVERSION.prefix] = TreeNOVERSION

class TreeDFREAD(TreeException):
    status = 265392234
    prefix = "%TREE-E-DFREAD"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-DFREAD, Error reading from datafile")

EXCEPTION_MAP[TreeDFREAD.status] = TreeDFREAD
EXCEPTION_PREFIX_MAP[TreeDFREAD.prefix] = TreeDFREAD

class TreeCLOSEERR(TreeException):
    status = 265392242
    prefix = "%TREE-E-CLOSEERR"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-CLOSEERR, Error closing temporary tree file")

EXCEPTION_MAP[TreeCLOSEERR.status] = TreeCLOSEERR
EXCEPTION_PREFIX_MAP[TreeCLOSEERR.prefix] = TreeCLOSEERR

class TreeMOVEERROR(TreeException):
    status = 265392250
    prefix = "%TREE-E-MOVEERROR"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-MOVEERROR, Error replacing original treefile with new one")

EXCEPTION_MAP[TreeMOVEERROR.status] = TreeMOVEERROR
EXCEPTION_PREFIX_MAP[TreeMOVEERROR.prefix] = TreeMOVEERROR

class TreeOPENEDITERR(TreeException):
    status = 265392258
    prefix = "%TREE-E-OPENEDITERR"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-OPENEDITERR, Error reopening new treefile for write access")

EXCEPTION_MAP[TreeOPENEDITERR.status] = TreeOPENEDITERR
EXCEPTION_PREFIX_MAP[TreeOPENEDITERR.prefix] = TreeOPENEDITERR

class TreeREADONLY_TREE(TreeException):
    status = 265392266
    prefix = "%TREE-E-READONLY_TREE"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-READONLY_TREE, Tree is marked as readonly. No write operations permitted")

EXCEPTION_MAP[TreeREADONLY_TREE.status] = TreeREADONLY_TREE
EXCEPTION_PREFIX_MAP[TreeREADONLY_TREE.prefix] = TreeREADONLY_TREE

class TreeWRITETREEERR(TreeException):
    status = 265392274
    prefix = "%TREE-E-WRITETREEERR"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-WRITETREEERR, Error writing .tree file")

EXCEPTION_MAP[TreeWRITETREEERR.status] = TreeWRITETREEERR
EXCEPTION_PREFIX_MAP[TreeWRITETREEERR.prefix] = TreeWRITETREEERR

class TreeNOWILD(TreeException):
    status = 265392282
    prefix = "%TREE-E-NOWILD"
    def __init__(self):
        Exception.__init__(self, "%TREE-E-NOWILD, No wildcard characters permitted in node specifier")

EXCEPTION_MAP[TreeNOWILD.status] = TreeNOWILD
EXCEPTION_PREFIX_MAP[TreeNOWILD.prefix] = TreeNOWILD

class LibException(MdsException):
    pass

class LibINSVIRMEM(LibException):
    status = 1409556
    prefix = "%LIB-F-INSVIRMEM"
    def __init__(self):
        Exception.__init__(self, "%LIB-F-INSVIRMEM, Insufficient virtual memory")

EXCEPTION_MAP[LibINSVIRMEM.status] = LibINSVIRMEM
EXCEPTION_PREFIX_MAP[LibINSVIRMEM.prefix] = LibINSVIRMEM

class LibINVARG(LibException):
    status = 1409588
    prefix = "%LIB-F-INVARG"
    def __init__(self):
        Exception.__init__(self, "%LIB-F-INVARG, Invalid argument")

EXCEPTION_MAP[LibINVARG.status] = LibINVARG
EXCEPTION_PREFIX_MAP[LibINVARG.prefix] = LibINVARG

class LibINVSTRDES(LibException):
    status = 1409572
    prefix = "%LIB-F-INVSTRDES"
    def __init__(self):
        Exception.__init__(self, "%LIB-F-INVSTRDES, Invalid string descriptor")

EXCEPTION_MAP[LibINVSTRDES.status] = LibINVSTRDES
EXCEPTION_PREFIX_MAP[LibINVSTRDES.prefix] = LibINVSTRDES

class LibKEYNOTFOU(LibException):
    status = 1409788
    prefix = "%LIB-F-KEYNOTFOU"
    def __init__(self):
        Exception.__init__(self, "%LIB-F-KEYNOTFOU, Key not found")

EXCEPTION_MAP[LibKEYNOTFOU.status] = LibKEYNOTFOU
EXCEPTION_PREFIX_MAP[LibKEYNOTFOU.prefix] = LibKEYNOTFOU

class LibNOTFOU(LibException):
    status = 1409652
    prefix = "%LIB-F-NOTFOU"
    def __init__(self):
        Exception.__init__(self, "%LIB-F-NOTFOU, Entity not found")

EXCEPTION_MAP[LibNOTFOU.status] = LibNOTFOU
EXCEPTION_PREFIX_MAP[LibNOTFOU.prefix] = LibNOTFOU

class LibQUEWASEMP(LibException):
    status = 1409772
    prefix = "%LIB-F-QUEWASEMP"
    def __init__(self):
        Exception.__init__(self, "%LIB-F-QUEWASEMP, Queue was empty")

EXCEPTION_MAP[LibQUEWASEMP.status] = LibQUEWASEMP
EXCEPTION_PREFIX_MAP[LibQUEWASEMP.prefix] = LibQUEWASEMP

class LibSTRTRU(LibException):
    status = 1409041
    prefix = "%LIB-S-STRTRU"
    def __init__(self):
        Exception.__init__(self, "%LIB-S-STRTRU, String truncated")

EXCEPTION_MAP[LibSTRTRU.status] = LibSTRTRU
EXCEPTION_PREFIX_MAP[LibSTRTRU.prefix] = LibSTRTRU

class StrException(MdsException):
    pass

class StrMATCH(StrException):
    status = 2393113
    prefix = "%STR-S-MATCH"
    def __init__(self):
        Exception.__init__(self, "%STR-S-MATCH, Strings match")

EXCEPTION_MAP[StrMATCH.status] = StrMATCH
EXCEPTION_PREFIX_MAP[StrMATCH.prefix] = StrMATCH

class StrNOMATCH(StrException):
    status = 2392584
    prefix = "%STR-W-NOMATCH"
    def __init__(self):
        Exception.__init__(self, "%STR-W-NOMATCH, Strings do not match")

EXCEPTION_MAP[StrNOMATCH.status] = StrNOMATCH
EXCEPTION_PREFIX_MAP[StrNOMATCH.prefix] = StrNOMATCH

class StrNOELEM(StrException):
    status = 2392600
    prefix = "%STR-W-NOELEM"
    def __init__(self):
        Exception.__init__(self, "%STR-W-NOELEM, Not enough delimited characters")

EXCEPTION_MAP[StrNOELEM.status] = StrNOELEM
EXCEPTION_PREFIX_MAP[StrNOELEM.prefix] = StrNOELEM

class StrINVDELIM(StrException):
    status = 2392592
    prefix = "%STR-W-INVDELIM"
    def __init__(self):
        Exception.__init__(self, "%STR-W-INVDELIM, Not enough delimited characters")

EXCEPTION_MAP[StrINVDELIM.status] = StrINVDELIM
EXCEPTION_PREFIX_MAP[StrINVDELIM.prefix] = StrINVDELIM

class StrSTRTOOLON(StrException):
    status = 2392180
    prefix = "%STR-F-STRTOOLON"
    def __init__(self):
        Exception.__init__(self, "%STR-F-STRTOOLON, String too long")

EXCEPTION_MAP[StrSTRTOOLON.status] = StrSTRTOOLON
EXCEPTION_PREFIX_MAP[StrSTRTOOLON.prefix] = StrSTRTOOLON

class MDSplusException(MdsException):
    pass

class MDSplusWARNING(MDSplusException):
    status = 65536
    prefix = "%MDSPLUS-W-WARNING"
    def __init__(self):
        Exception.__init__(self, "%MDSPLUS-W-WARNING, Warning")

EXCEPTION_MAP[MDSplusWARNING.status] = MDSplusWARNING
EXCEPTION_PREFIX_MAP[MDSplusWARNING.prefix] = MDSplusWARNING

class MDSplusSUCCESS(MDSplusException):
    status = 65545
    prefix = "%MDSPLUS-S-SUCCESS"
    def __init__(self):
        Exception.__init__(self, "%MDSPLUS-S-SUCCESS, Success")

EXCEPTION_MAP[MDSplusSUCCESS.status] = MDSplusSUCCESS
EXCEPTION_PREFIX_MAP[MDSplusSUCCESS.prefix] = MDSplusSUCCESS

class MDSplusERROR(MDSplusException):
    status = 65554
    prefix = "%MDSPLUS-E-ERROR"
    def __init__(self):
        Exception.__init__(self, "%MDSPLUS-E-ERROR, Error")

EXCEPTION_MAP[MDSplusERROR.status] = MDSplusERROR
EXCEPTION_PREFIX_MAP[MDSplusERROR.prefix] = MDSplusERROR

class MDSplusFATAL(MDSplusException):
    status = 65572
    prefix = "%MDSPLUS-F-FATAL"
    def __init__(self):
        Exception.__init__(self, "%MDSPLUS-F-FATAL, Fatal")

EXCEPTION_MAP[MDSplusFATAL.status] = MDSplusFATAL
EXCEPTION_PREFIX_MAP[MDSplusFATAL.prefix] = MDSplusFATAL

class MDSplusSANDBOX(MDSplusException):
    status = 65578
    prefix = "%MDSPLUS-E-SANDBOX"
    def __init__(self):
        Exception.__init__(self, "%MDSPLUS-E-SANDBOX, Function disabled for security reasons")

EXCEPTION_MAP[MDSplusSANDBOX.status] = MDSplusSANDBOX
EXCEPTION_PREFIX_MAP[MDSplusSANDBOX.prefix] = MDSplusSANDBOX

class TdiException(MdsException):
    pass

class TdiBREAK(TdiException):
    status = 265519112
    prefix = "%TDI-W-BREAK"
    def __init__(self):
        Exception.__init__(self, "%TDI-W-BREAK, BREAK was not in DO FOR SWITCH or WHILE")

EXCEPTION_MAP[TdiBREAK.status] = TdiBREAK
EXCEPTION_PREFIX_MAP[TdiBREAK.prefix] = TdiBREAK

class TdiCASE(TdiException):
    status = 265519120
    prefix = "%TDI-W-CASE"
    def __init__(self):
        Exception.__init__(self, "%TDI-W-CASE, CASE was not in SWITCH statement")

EXCEPTION_MAP[TdiCASE.status] = TdiCASE
EXCEPTION_PREFIX_MAP[TdiCASE.prefix] = TdiCASE

class TdiCONTINUE(TdiException):
    status = 265519128
    prefix = "%TDI-W-CONTINUE"
    def __init__(self):
        Exception.__init__(self, "%TDI-W-CONTINUE, CONTINUE was not in DO FOR or WHILE")

EXCEPTION_MAP[TdiCONTINUE.status] = TdiCONTINUE
EXCEPTION_PREFIX_MAP[TdiCONTINUE.prefix] = TdiCONTINUE

class TdiEXTRANEOUS(TdiException):
    status = 265519136
    prefix = "%TDI-W-EXTRANEOUS"
    def __init__(self):
        Exception.__init__(self, "%TDI-W-EXTRANEOUS, Some characters were unused, bad number maybe")

EXCEPTION_MAP[TdiEXTRANEOUS.status] = TdiEXTRANEOUS
EXCEPTION_PREFIX_MAP[TdiEXTRANEOUS.prefix] = TdiEXTRANEOUS

class TdiRETURN(TdiException):
    status = 265519144
    prefix = "%TDI-W-RETURN"
    def __init__(self):
        Exception.__init__(self, "%TDI-W-RETURN, Extraneous RETURN statement, not from a FUN")

EXCEPTION_MAP[TdiRETURN.status] = TdiRETURN
EXCEPTION_PREFIX_MAP[TdiRETURN.prefix] = TdiRETURN

class TdiABORT(TdiException):
    status = 265519154
    prefix = "%TDI-E-ABORT"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-ABORT, Program requested abort")

EXCEPTION_MAP[TdiABORT.status] = TdiABORT
EXCEPTION_PREFIX_MAP[TdiABORT.prefix] = TdiABORT

class TdiBAD_INDEX(TdiException):
    status = 265519162
    prefix = "%TDI-E-BAD_INDEX"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-BAD_INDEX, Index or subscript is too small or too big")

EXCEPTION_MAP[TdiBAD_INDEX.status] = TdiBAD_INDEX
EXCEPTION_PREFIX_MAP[TdiBAD_INDEX.prefix] = TdiBAD_INDEX

class TdiBOMB(TdiException):
    status = 265519170
    prefix = "%TDI-E-BOMB"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-BOMB, Bad punctuation, could not compile the text")

EXCEPTION_MAP[TdiBOMB.status] = TdiBOMB
EXCEPTION_PREFIX_MAP[TdiBOMB.prefix] = TdiBOMB

class TdiEXTRA_ARG(TdiException):
    status = 265519178
    prefix = "%TDI-E-EXTRA_ARG"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-EXTRA_ARG, Too many arguments for function, watch commas")

EXCEPTION_MAP[TdiEXTRA_ARG.status] = TdiEXTRA_ARG
EXCEPTION_PREFIX_MAP[TdiEXTRA_ARG.prefix] = TdiEXTRA_ARG

class TdiGOTO(TdiException):
    status = 265519186
    prefix = "%TDI-E-GOTO"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-GOTO, GOTO target label not found")

EXCEPTION_MAP[TdiGOTO.status] = TdiGOTO
EXCEPTION_PREFIX_MAP[TdiGOTO.prefix] = TdiGOTO

class TdiINVCLADSC(TdiException):
    status = 265519194
    prefix = "%TDI-E-INVCLADSC"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-INVCLADSC, Storage class not valid, must be scalar or array")

EXCEPTION_MAP[TdiINVCLADSC.status] = TdiINVCLADSC
EXCEPTION_PREFIX_MAP[TdiINVCLADSC.prefix] = TdiINVCLADSC

class TdiINVCLADTY(TdiException):
    status = 265519202
    prefix = "%TDI-E-INVCLADTY"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-INVCLADTY, Invalid mixture of storage class and data type")

EXCEPTION_MAP[TdiINVCLADTY.status] = TdiINVCLADTY
EXCEPTION_PREFIX_MAP[TdiINVCLADTY.prefix] = TdiINVCLADTY

class TdiINVDTYDSC(TdiException):
    status = 265519210
    prefix = "%TDI-E-INVDTYDSC"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-INVDTYDSC, Storage data type is not valid")

EXCEPTION_MAP[TdiINVDTYDSC.status] = TdiINVDTYDSC
EXCEPTION_PREFIX_MAP[TdiINVDTYDSC.prefix] = TdiINVDTYDSC

class TdiINV_OPC(TdiException):
    status = 265519218
    prefix = "%TDI-E-INV_OPC"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-INV_OPC, Invalid operator code in a function")

EXCEPTION_MAP[TdiINV_OPC.status] = TdiINV_OPC
EXCEPTION_PREFIX_MAP[TdiINV_OPC.prefix] = TdiINV_OPC

class TdiINV_SIZE(TdiException):
    status = 265519226
    prefix = "%TDI-E-INV_SIZE"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-INV_SIZE, Number of elements does not match declaration")

EXCEPTION_MAP[TdiINV_SIZE.status] = TdiINV_SIZE
EXCEPTION_PREFIX_MAP[TdiINV_SIZE.prefix] = TdiINV_SIZE

class TdiMISMATCH(TdiException):
    status = 265519234
    prefix = "%TDI-E-MISMATCH"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-MISMATCH, Shape of arguments does not match")

EXCEPTION_MAP[TdiMISMATCH.status] = TdiMISMATCH
EXCEPTION_PREFIX_MAP[TdiMISMATCH.prefix] = TdiMISMATCH

class TdiMISS_ARG(TdiException):
    status = 265519242
    prefix = "%TDI-E-MISS_ARG"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-MISS_ARG, Missing argument is required for function")

EXCEPTION_MAP[TdiMISS_ARG.status] = TdiMISS_ARG
EXCEPTION_PREFIX_MAP[TdiMISS_ARG.prefix] = TdiMISS_ARG

class TdiNDIM_OVER(TdiException):
    status = 265519250
    prefix = "%TDI-E-NDIM_OVER"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-NDIM_OVER, Number of dimensions is over the allowed 8")

EXCEPTION_MAP[TdiNDIM_OVER.status] = TdiNDIM_OVER
EXCEPTION_PREFIX_MAP[TdiNDIM_OVER.prefix] = TdiNDIM_OVER

class TdiNO_CMPLX(TdiException):
    status = 265519258
    prefix = "%TDI-E-NO_CMPLX"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-NO_CMPLX, There are no complex forms of this function")

EXCEPTION_MAP[TdiNO_CMPLX.status] = TdiNO_CMPLX
EXCEPTION_PREFIX_MAP[TdiNO_CMPLX.prefix] = TdiNO_CMPLX

class TdiNO_OPC(TdiException):
    status = 265519266
    prefix = "%TDI-E-NO_OPC"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-NO_OPC, No support for this function, today")

EXCEPTION_MAP[TdiNO_OPC.status] = TdiNO_OPC
EXCEPTION_PREFIX_MAP[TdiNO_OPC.prefix] = TdiNO_OPC

class TdiNO_OUTPTR(TdiException):
    status = 265519274
    prefix = "%TDI-E-NO_OUTPTR"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-NO_OUTPTR, An output pointer is required")

EXCEPTION_MAP[TdiNO_OUTPTR.status] = TdiNO_OUTPTR
EXCEPTION_PREFIX_MAP[TdiNO_OUTPTR.prefix] = TdiNO_OUTPTR

class TdiNO_SELF_PTR(TdiException):
    status = 265519282
    prefix = "%TDI-E-NO_SELF_PTR"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-NO_SELF_PTR, No $VALUE is defined for signal or validation")

EXCEPTION_MAP[TdiNO_SELF_PTR.status] = TdiNO_SELF_PTR
EXCEPTION_PREFIX_MAP[TdiNO_SELF_PTR.prefix] = TdiNO_SELF_PTR

class TdiNOT_NUMBER(TdiException):
    status = 265519290
    prefix = "%TDI-E-NOT_NUMBER"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-NOT_NUMBER, Value is not a scalar number and must be")

EXCEPTION_MAP[TdiNOT_NUMBER.status] = TdiNOT_NUMBER
EXCEPTION_PREFIX_MAP[TdiNOT_NUMBER.prefix] = TdiNOT_NUMBER

class TdiNULL_PTR(TdiException):
    status = 265519298
    prefix = "%TDI-E-NULL_PTR"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-NULL_PTR, Null pointer where value needed")

EXCEPTION_MAP[TdiNULL_PTR.status] = TdiNULL_PTR
EXCEPTION_PREFIX_MAP[TdiNULL_PTR.prefix] = TdiNULL_PTR

class TdiRECURSIVE(TdiException):
    status = 265519306
    prefix = "%TDI-E-RECURSIVE"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-RECURSIVE, Overly recursive function, calls itself maybe")

EXCEPTION_MAP[TdiRECURSIVE.status] = TdiRECURSIVE
EXCEPTION_PREFIX_MAP[TdiRECURSIVE.prefix] = TdiRECURSIVE

class TdiSIG_DIM(TdiException):
    status = 265519314
    prefix = "%TDI-E-SIG_DIM"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-SIG_DIM, Signal dimension does not match data shape")

EXCEPTION_MAP[TdiSIG_DIM.status] = TdiSIG_DIM
EXCEPTION_PREFIX_MAP[TdiSIG_DIM.prefix] = TdiSIG_DIM

class TdiSYNTAX(TdiException):
    status = 265519322
    prefix = "%TDI-E-SYNTAX"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-SYNTAX, Bad punctuation or misspelled word or number")

EXCEPTION_MAP[TdiSYNTAX.status] = TdiSYNTAX
EXCEPTION_PREFIX_MAP[TdiSYNTAX.prefix] = TdiSYNTAX

class TdiTOO_BIG(TdiException):
    status = 265519330
    prefix = "%TDI-E-TOO_BIG"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-TOO_BIG, Conversion of number lost significant digits")

EXCEPTION_MAP[TdiTOO_BIG.status] = TdiTOO_BIG
EXCEPTION_PREFIX_MAP[TdiTOO_BIG.prefix] = TdiTOO_BIG

class TdiUNBALANCE(TdiException):
    status = 265519338
    prefix = "%TDI-E-UNBALANCE"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-UNBALANCE, Unbalanced () [] {} '' "" or /**/")

EXCEPTION_MAP[TdiUNBALANCE.status] = TdiUNBALANCE
EXCEPTION_PREFIX_MAP[TdiUNBALANCE.prefix] = TdiUNBALANCE

class TdiUNKNOWN_VAR(TdiException):
    status = 265519346
    prefix = "%TDI-E-UNKNOWN_VAR"
    def __init__(self):
        Exception.__init__(self, "%TDI-E-UNKNOWN_VAR, Unknown/undefined variable name")

EXCEPTION_MAP[TdiUNKNOWN_VAR.status] = TdiUNKNOWN_VAR
EXCEPTION_PREFIX_MAP[TdiUNKNOWN_VAR.prefix] = TdiUNKNOWN_VAR

class TdiSTRTOOLON(TdiException):
    status = 265519356
    prefix = "%TDI-F-STRTOOLON"
    def __init__(self):
        Exception.__init__(self, "%TDI-F-STRTOOLON, String is too long (greater than 65535)")

EXCEPTION_MAP[TdiSTRTOOLON.status] = TdiSTRTOOLON
EXCEPTION_PREFIX_MAP[TdiSTRTOOLON.prefix] = TdiSTRTOOLON

class TdiTIMEOUT(TdiException):
    status = 265519364
    prefix = "%TDI-F-TIMEOUT"
    def __init__(self):
        Exception.__init__(self, "%TDI-F-TIMEOUT, Task did not complete in alotted time")

EXCEPTION_MAP[TdiTIMEOUT.status] = TdiTIMEOUT
EXCEPTION_PREFIX_MAP[TdiTIMEOUT.prefix] = TdiTIMEOUT

class ApdException(MdsException):
    pass

class ApdAPD_APPEND(ApdException):
    status = 266141706
    prefix = "%APD-E-APD_APPEND"
    def __init__(self):
        Exception.__init__(self, "%APD-E-APD_APPEND, First argument must be APD or *")

EXCEPTION_MAP[ApdAPD_APPEND.status] = ApdAPD_APPEND
EXCEPTION_PREFIX_MAP[ApdAPD_APPEND.prefix] = ApdAPD_APPEND

class ApdDICT_KEYVALPAIR(ApdException):
    status = 266141714
    prefix = "%APD-E-DICT_KEYVALPAIR"
    def __init__(self):
        Exception.__init__(self, "%APD-E-DICT_KEYVALPAIR, A Dictionary requires an even number of elements")

EXCEPTION_MAP[ApdDICT_KEYVALPAIR.status] = ApdDICT_KEYVALPAIR
EXCEPTION_PREFIX_MAP[ApdDICT_KEYVALPAIR.prefix] = ApdDICT_KEYVALPAIR

class ApdDICT_KEYCLS(ApdException):
    status = 266141722
    prefix = "%APD-E-DICT_KEYCLS"
    def __init__(self):
        Exception.__init__(self, "%APD-E-DICT_KEYCLS, Keys must be scalar, i.e. CLASS_S")

EXCEPTION_MAP[ApdDICT_KEYCLS.status] = ApdDICT_KEYCLS
EXCEPTION_PREFIX_MAP[ApdDICT_KEYCLS.prefix] = ApdDICT_KEYCLS

class TclException(MdsException):
    pass

class TclNORMAL(TclException):
    status = 2752521
    prefix = "%TCL-S-NORMAL"
    def __init__(self):
        Exception.__init__(self, "%TCL-S-NORMAL, Normal successful completion")

EXCEPTION_MAP[TclNORMAL.status] = TclNORMAL
EXCEPTION_PREFIX_MAP[TclNORMAL.prefix] = TclNORMAL

class TclFAILED_ESSENTIAL(TclException):
    status = 2752528
    prefix = "%TCL-W-FAILED_ESSENTIAL"
    def __init__(self):
        Exception.__init__(self, "%TCL-W-FAILED_ESSENTIAL, Essential action failed")

EXCEPTION_MAP[TclFAILED_ESSENTIAL.status] = TclFAILED_ESSENTIAL
EXCEPTION_PREFIX_MAP[TclFAILED_ESSENTIAL.prefix] = TclFAILED_ESSENTIAL

class TclNO_DISPATCH_TABLE(TclException):
    status = 2752536
    prefix = "%TCL-W-NO_DISPATCH_TABLE"
    def __init__(self):
        Exception.__init__(self, "%TCL-W-NO_DISPATCH_TABLE, No dispatch table found. Forgot to do DISPATCH/BUILD?")

EXCEPTION_MAP[TclNO_DISPATCH_TABLE.status] = TclNO_DISPATCH_TABLE
EXCEPTION_PREFIX_MAP[TclNO_DISPATCH_TABLE.prefix] = TclNO_DISPATCH_TABLE




#
# Copyright (c) 2024, Massachusetts Institute of Technology All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
#
# Redistributions in binary form must reproduce the above copyright notice, this
# list of conditions and the following disclaimer in the documentation and/or
# other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

OPCODE_MAP = {
    '$': 0,
    '$A0': 1,
    '$ALPHA': 2,
    '$AMU': 3,
    '$C': 4,
    '$CAL': 5,
    '$DEGREE': 6,
    '$EV': 7,
    '$FALSE': 8,
    '$FARADAY': 9,
    '$G': 10,
    '$GAS': 11,
    '$H': 12,
    '$HBAR': 13,
    '$I': 14,
    '$K': 15,
    '$ME': 16,
    '$MISSING': 17,
    '$MP': 18,
    '$N0': 19,
    '$NA': 20,
    '$P0': 21,
    '$PI': 22,
    '$QE': 23,
    '$RE': 24,
    '$ROPRAND': 25,
    '$RYDBERG': 26,
    '$T0': 27,
    '$TORR': 28,
    '$TRUE': 29,
    '$VALUE': 30,
    'ABORT': 31,
    'ABS': 32,
    'ABS1': 33,
    'ABSSQ': 34,
    'ACHAR': 35,
    'ACOS': 36,
    'ACOSD': 37,
    'ADD': 38,
    'ADJUSTL': 39,
    'ADJUSTR': 40,
    'AIMAG': 41,
    'AINT': 42,
    'ALL': 43,
    'ALLOCATED': 44,
    'AND': 45,
    'AND_NOT': 46,
    'ANINT': 47,
    'ANY': 48,
    'ARG': 49,
    'ARGD': 50,
    'ARG_OF': 51,
    'ARRAY': 52,
    'ASIN': 53,
    'ASIND': 54,
    'AS_IS': 55,
    'ATAN': 56,
    'ATAN2': 57,
    'ATAN2D': 58,
    'ATAND': 59,
    'ATANH': 60,
    'AXIS_OF': 61,
    'BACKSPACE': 62,
    'IBCLR': 63,
    'BEGIN_OF': 64,
    'IBITS': 65,
    'BREAK': 66,
    'BSEARCH': 67,
    'IBSET': 68,
    'BTEST': 69,
    'BUILD_ACTION': 70,
    'BUILD_CONDITION': 71,
    'BUILD_CONGLOM': 72,
    'BUILD_DEPENDENCY': 73,
    'BUILD_DIM': 74,
    'BUILD_DISPATCH': 75,
    'BUILD_EVENT': 76,
    'BUILD_FUNCTION': 77,
    'BUILD_METHOD': 78,
    'BUILD_PARAM': 79,
    'BUILD_PATH': 80,
    'BUILD_PROCEDURE': 81,
    'BUILD_PROGRAM': 82,
    'BUILD_RANGE': 83,
    'BUILD_ROUTINE': 84,
    'BUILD_SIGNAL': 85,
    'BUILD_SLOPE': 86,
    'BUILD_WINDOW': 87,
    'BUILD_WITH_UNITS': 88,
    'BUILTIN_OPCODE': 89,
    'BYTE': 90,
    'BYTE_UNSIGNED': 91,
    'CASE': 92,
    'CEILING': 93,
    'CHAR': 94,
    'CLASS': 95,
    'FCLOSE': 96,
    'CMPLX': 97,
    'COMMA': 98,
    'COMPILE': 99,
    'COMPLETION_OF': 100,
    'CONCAT': 101,
    'CONDITIONAL': 102,
    'CONJG': 103,
    'CONTINUE': 104,
    'CONVOLVE': 105,
    'COS': 106,
    'COSD': 107,
    'COSH': 108,
    'COUNT': 109,
    'CSHIFT': 110,
    'CVT': 111,
    'DATA': 112,
    'DATE_AND_TIME': 113,
    'DATE_TIME': 114,
    'DBLE': 115,
    'DEALLOCATE': 116,
    'DEBUG': 117,
    'DECODE': 118,
    'DECOMPILE': 119,
    'DECOMPRESS': 120,
    'DEFAULT': 121,
    'DERIVATIVE': 122,
    'DESCR': 123,
    'DIAGONAL': 124,
    'DIGITS': 125,
    'DIM': 126,
    'DIM_OF': 127,
    'DISPATCH_OF': 128,
    'DIVIDE': 129,
    'LBOUND': 130,
    'DO': 131,
    'DOT_PRODUCT': 132,
    'DPROD': 133,
    'DSCPTR': 134,
    'SHAPE': 135,
    'SIZE': 136,
    'KIND': 137,
    'UBOUND': 138,
    'D_COMPLEX': 139,
    'D_FLOAT': 140,
    'RANGE': 141,
    'PRECISION': 142,
    'ELBOUND': 143,
    'ELSE': 144,
    'ELSEWHERE': 145,
    'ENCODE': 146,
    'ENDFILE': 147,
    'END_OF': 148,
    'EOSHIFT': 149,
    'EPSILON': 150,
    'EQ': 151,
    'EQUALS': 152,
    'EQUALS_FIRST': 153,
    'EQV': 154,
    'ESHAPE': 155,
    'ESIZE': 156,
    'EUBOUND': 157,
    'EVALUATE': 158,
    'EXECUTE': 159,
    'EXP': 160,
    'EXPONENT': 161,
    'EXT_FUNCTION': 162,
    'FFT': 163,
    'FIRSTLOC': 164,
    'FIT': 165,
    'FIX_ROPRAND': 166,
    'FLOAT': 167,
    'FLOOR': 168,
    'FOR': 169,
    'FRACTION': 170,
    'FUN': 171,
    'F_COMPLEX': 172,
    'F_FLOAT': 173,
    'GE': 174,
    'GETNCI': 175,
    'GOTO': 176,
    'GT': 177,
    'G_COMPLEX': 178,
    'G_FLOAT': 179,
    'HELP_OF': 180,
    'HUGE': 181,
    'H_COMPLEX': 182,
    'H_FLOAT': 183,
    'IACHAR': 184,
    'IAND': 185,
    'IAND_NOT': 186,
    'ICHAR': 187,
    'IDENT_OF': 188,
    'IF': 189,
    'IF_ERROR': 190,
    'IMAGE_OF': 191,
    'IN': 192,
    'INAND': 193,
    'INAND_NOT': 194,
    'INDEX': 195,
    'INOR': 196,
    'INOR_NOT': 197,
    'INOT': 198,
    'INOUT': 199,
    'INQUIRE': 200,
    'INT': 201,
    'INTEGRAL': 202,
    'INTERPOL': 203,
    'INTERSECT': 204,
    'INT_UNSIGNED': 205,
    'INVERSE': 206,
    'IOR': 207,
    'IOR_NOT': 208,
    'IS_IN': 209,
    'IEOR': 210,
    'IEOR_NOT': 211,
    'LABEL': 212,
    'LAMINATE': 213,
    'LANGUAGE_OF': 214,
    'LASTLOC': 215,
    'LE': 216,
    'LEN': 217,
    'LEN_TRIM': 218,
    'LGE': 219,
    'LGT': 220,
    'LLE': 221,
    'LLT': 222,
    'LOG': 223,
    'LOG10': 224,
    'LOG2': 225,
    'LOGICAL': 226,
    'LONG': 227,
    'LONG_UNSIGNED': 228,
    'LT': 229,
    'MATMUL': 230,
    'MAT_ROT': 231,
    'MAT_ROT_INT': 232,
    'MAX': 233,
    'MAXEXPONENT': 234,
    'MAXLOC': 235,
    'MAXVAL': 236,
    'MEAN': 237,
    'MEDIAN': 238,
    'MERGE': 239,
    'METHOD_OF': 240,
    'MIN': 241,
    'MINEXPONENT': 242,
    'MINLOC': 243,
    'MINVAL': 244,
    'MOD': 245,
    'MODEL_OF': 246,
    'MULTIPLY': 247,
    'NAME_OF': 248,
    'NAND': 249,
    'NAND_NOT': 250,
    'NDESC': 251,
    'NE': 252,
    'NEAREST': 253,
    'NEQV': 254,
    'NINT': 255,
    'NOR': 256,
    'NOR_NOT': 257,
    'NOT': 258,
    'OBJECT_OF': 259,
    'OCTAWORD': 260,
    'OCTAWORD_UNSIGNED': 261,
    'ON_ERROR': 262,
    'OPCODE_BUILTIN': 263,
    'OPCODE_STRING': 264,
    'FOPEN': 265,
    'OPTIONAL': 266,
    'OR': 267,
    'OR_NOT': 268,
    'OUT': 269,
    'PACK': 270,
    'PHASE_OF': 271,
    'POST_DEC': 272,
    'POST_INC': 273,
    'POWER': 274,
    'PRESENT': 275,
    'PRE_DEC': 276,
    'PRE_INC': 277,
    'PRIVATE': 278,
    'PROCEDURE_OF': 279,
    'PRODUCT': 280,
    'PROGRAM_OF': 281,
    'PROJECT': 282,
    'PROMOTE': 283,
    'PUBLIC': 284,
    'QUADWORD': 285,
    'QUADWORD_UNSIGNED': 286,
    'QUALIFIERS_OF': 287,
    'RADIX': 288,
    'RAMP': 289,
    'RANDOM': 290,
    'RANDOM_SEED': 291,
    'DTYPE_RANGE': 292,
    'RANK': 293,
    'RAW_OF': 294,
    'READ': 295,
    'REAL': 296,
    'REBIN': 297,
    'REF': 298,
    'REPEAT': 299,
    'REPLICATE': 300,
    'RESHAPE': 301,
    'RETURN': 302,
    'REWIND': 303,
    'RMS': 304,
    'ROUTINE_OF': 305,
    'RRSPACING': 306,
    'SCALE': 307,
    'SCAN': 308,
    'FSEEK': 309,
    'SET_EXPONENT': 310,
    'SET_RANGE': 311,
    'ISHFT': 312,
    'ISHFTC': 313,
    'SHIFT_LEFT': 314,
    'SHIFT_RIGHT': 315,
    'SIGN': 316,
    'SIGNED': 317,
    'SIN': 318,
    'SIND': 319,
    'SINH': 320,
    'SIZEOF': 321,
    'SLOPE_OF': 322,
    'SMOOTH': 323,
    'SOLVE': 324,
    'SORTVAL': 325,
    'SPACING': 326,
    'SPAWN': 327,
    'SPREAD': 328,
    'SQRT': 329,
    'SQUARE': 330,
    'STATEMENT': 331,
    'STD_DEV': 332,
    'STRING': 333,
    'STRING_OPCODE': 334,
    'SUBSCRIPT': 335,
    'SUBTRACT': 336,
    'SUM': 337,
    'SWITCH': 338,
    'SYSTEM_CLOCK': 339,
    'TAN': 340,
    'TAND': 341,
    'TANH': 342,
    'TASK_OF': 343,
    'TEXT': 344,
    'TIME_OUT_OF': 345,
    'TINY': 346,
    'TRANSFER': 347,
    'TRANSPOSE_': 348,
    'TRIM': 349,
    'UNARY_MINUS': 350,
    'UNARY_PLUS': 351,
    'UNION': 352,
    'UNITS': 353,
    'UNITS_OF': 354,
    'UNPACK': 355,
    'UNSIGNED': 356,
    'VAL': 357,
    'VALIDATION_OF': 358,
    'VALUE_OF': 359,
    'VAR': 360,
    'VECTOR': 361,
    'VERIFY': 362,
    'WAIT': 363,
    'WHEN_OF': 364,
    'WHERE': 365,
    'WHILE': 366,
    'WINDOW_OF': 367,
    'WORD': 368,
    'WORD_UNSIGNED': 369,
    'WRITE': 370,
    'ZERO': 371,
    '$2PI': 372,
    '$NARG': 373,
    'ELEMENT': 374,
    'RC_DROOP': 375,
    'RESET_PRIVATE': 376,
    'RESET_PUBLIC': 377,
    'SHOW_PRIVATE': 378,
    'SHOW_PUBLIC': 379,
    'SHOW_VM': 380,
    'TRANSLATE': 381,
    'TRANSPOSE_MUL': 382,
    'UPCASE': 383,
    'USING': 384,
    'VALIDATION': 385,
    '$DEFAULT': 386,
    '$EXPT': 387,
    '$SHOT': 388,
    'GETDBI': 389,
    'CULL': 390,
    'EXTEND': 391,
    'I_TO_X': 392,
    'X_TO_I': 393,
    'MAP': 394,
    'COMPILE_DEPENDENCY': 395,
    'DECOMPILE_DEPENDENCY': 396,
    'BUILD_CALL': 397,
    'ERRORLOGS_OF': 398,
    'PERFORMANCE_OF': 399,
    'XD': 400,
    'CONDITION_OF': 401,
    'SORT': 402,
    '$THIS': 403,
    'DATA_WITH_UNITS': 404,
    '$ATM': 405,
    '$EPSILON0': 406,
    '$GN': 407,
    '$MU0': 408,
    'EXTRACT': 409,
    'FINITE': 410,
    'BIT_SIZE': 411,
    'MODULO': 412,
    'SELECTED_INT_KIND': 413,
    'SELECTED_REAL_KIND': 414,
    'DSQL': 415,
    'ISQL': 416,
    'FTELL': 417,
    'MAKE_ACTION': 418,
    'MAKE_CONDITION': 419,
    'MAKE_CONGLOM': 420,
    'MAKE_DEPENDENCY': 421,
    'MAKE_DIM': 422,
    'MAKE_DISPATCH': 423,
    'MAKE_FUNCTION': 424,
    'MAKE_METHOD': 425,
    'MAKE_PARAM': 426,
    'MAKE_PROCEDURE': 427,
    'MAKE_PROGRAM': 428,
    'MAKE_RANGE': 429,
    'MAKE_ROUTINE': 430,
    'MAKE_SIGNAL': 431,
    'MAKE_WINDOW': 432,
    'MAKE_WITH_UNITS': 433,
    'MAKE_CALL': 434,
    'CLASS_OF': 435,
    'DSCPTR_OF': 436,
    'KIND_OF': 437,
    'NDESC_OF': 438,
    'ACCUMULATE': 439,
    'MAKE_SLOPE': 440,
    'REM': 441,
    'COMPLETION_MESSAGE_OF': 442,
    'INTERRUPT_OF': 443,
    '$SHOTNAME': 444,
    'BUILD_WITH_ERROR': 445,
    'ERROR_OF': 446,
    'MAKE_WITH_ERROR': 447,
    'DO_TASK': 448,
    'ISQL_SET': 449,
    'FS_FLOAT': 450,
    'FS_COMPLEX': 451,
    'FT_FLOAT': 452,
    'FT_COMPLEX': 453,
    'BUILD_OPAQUE': 454,
    'MAKE_OPAQUE': 455,
    'DICT': 456,
    'TUPLE': 457,
    'LIST': 458,
    'SQUEEZE': 459,
    'FLATTEN': 460,
}

OPCODE_REPR_MAP = {
    0: lambda: '$',
    1: lambda: '$A0',
    2: lambda: '$ALPHA',
    3: lambda: '$AMU',
    4: lambda: '$C',
    5: lambda: '$CAL',
    6: lambda: '$DEGREE',
    7: lambda: '$EV',
    8: lambda: '$FALSE',
    9: lambda: '$FARADAY',
    10: lambda: '$G',
    11: lambda: '$GAS',
    12: lambda: '$H',
    13: lambda: '$HBAR',
    14: lambda: '$I',
    15: lambda: '$K',
    16: lambda: '$ME',
    17: lambda: '$MISSING',
    18: lambda: '$MP',
    19: lambda: '$N0',
    20: lambda: '$NA',
    21: lambda: '$P0',
    22: lambda: '$PI',
    23: lambda: '$QE',
    24: lambda: '$RE',
    25: lambda: '$ROPRAND',
    26: lambda: '$RYDBERG',
    27: lambda: '$T0',
    28: lambda: '$TORR',
    29: lambda: '$TRUE',
    30: lambda: '$VALUE',
    31: lambda *args: f'ABORT({", ".join(map(repr, args))})',
    32: lambda *args: f'ABS({", ".join(map(repr, args))})',
    33: lambda *args: f'ABS1({", ".join(map(repr, args))})',
    34: lambda *args: f'ABSSQ({", ".join(map(repr, args))})',
    35: lambda *args: f'ACHAR({", ".join(map(repr, args))})',
    36: lambda *args: f'ACOS({", ".join(map(repr, args))})',
    37: lambda *args: f'ACOSD({", ".join(map(repr, args))})',
    38: lambda a,b: f"({repr(a)} + {repr(b)})",
    39: lambda *args: f'ADJUSTL({", ".join(map(repr, args))})',
    40: lambda *args: f'ADJUSTR({", ".join(map(repr, args))})',
    41: lambda *args: f'AIMAG({", ".join(map(repr, args))})',
    42: lambda *args: f'AINT({", ".join(map(repr, args))})',
    43: lambda *args: f'ALL({", ".join(map(repr, args))})',
    44: lambda *args: f'ALLOCATED({", ".join(map(repr, args))})',
    45: lambda *args: f'AND({", ".join(map(repr, args))})',
    46: lambda *args: f'AND_NOT({", ".join(map(repr, args))})',
    47: lambda *args: f'ANINT({", ".join(map(repr, args))})',
    48: lambda *args: f'ANY({", ".join(map(repr, args))})',
    49: lambda *args: f'ARG({", ".join(map(repr, args))})',
    50: lambda *args: f'ARGD({", ".join(map(repr, args))})',
    51: lambda *args: f'ARG_OF({", ".join(map(repr, args))})',
    52: lambda *args: f'ARRAY({", ".join(map(repr, args))})',
    53: lambda *args: f'ASIN({", ".join(map(repr, args))})',
    54: lambda *args: f'ASIND({", ".join(map(repr, args))})',
    55: lambda *args: f'AS_IS({", ".join(map(repr, args))})',
    56: lambda *args: f'ATAN({", ".join(map(repr, args))})',
    57: lambda *args: f'ATAN2({", ".join(map(repr, args))})',
    58: lambda *args: f'ATAN2D({", ".join(map(repr, args))})',
    59: lambda *args: f'ATAND({", ".join(map(repr, args))})',
    60: lambda *args: f'ATANH({", ".join(map(repr, args))})',
    61: lambda *args: f'AXIS_OF({", ".join(map(repr, args))})',
    62: lambda *args: f'BACKSPACE({", ".join(map(repr, args))})',
    63: lambda *args: f'IBCLR({", ".join(map(repr, args))})',
    64: lambda *args: f'BEGIN_OF({", ".join(map(repr, args))})',
    65: lambda *args: f'IBITS({", ".join(map(repr, args))})',
    66: lambda: 'BREAK',
    67: lambda *args: f'BSEARCH({", ".join(map(repr, args))})',
    68: lambda *args: f'IBSET({", ".join(map(repr, args))})',
    69: lambda *args: f'BTEST({", ".join(map(repr, args))})',
    70: lambda *args: f'BUILD_ACTION({", ".join(map(repr, args))})',
    71: lambda *args: f'BUILD_CONDITION({", ".join(map(repr, args))})',
    72: lambda *args: f'BUILD_CONGLOM({", ".join(map(repr, args))})',
    73: lambda *args: f'BUILD_DEPENDENCY({", ".join(map(repr, args))})',
    74: lambda *args: f'BUILD_DIM({", ".join(map(repr, args))})',
    75: lambda *args: f'BUILD_DISPATCH({", ".join(map(repr, args))})',
    76: lambda *args: f'BUILD_EVENT({", ".join(map(repr, args))})',
    77: lambda *args: f'BUILD_FUNCTION({", ".join(map(repr, args))})',
    78: lambda *args: f'BUILD_METHOD({", ".join(map(repr, args))})',
    79: lambda *args: f'BUILD_PARAM({", ".join(map(repr, args))})',
    80: lambda *args: f'BUILD_PATH({", ".join(map(repr, args))})',
    81: lambda *args: f'BUILD_PROCEDURE({", ".join(map(repr, args))})',
    82: lambda *args: f'BUILD_PROGRAM({", ".join(map(repr, args))})',
    83: lambda *args: f'BUILD_RANGE({", ".join(map(repr, args))})',
    84: lambda *args: f'BUILD_ROUTINE({", ".join(map(repr, args))})',
    85: lambda *args: f'BUILD_SIGNAL({", ".join(map(repr, args))})',
    86: lambda *args: f'BUILD_SLOPE({", ".join(map(repr, args))})',
    87: lambda *args: f'BUILD_WINDOW({", ".join(map(repr, args))})',
    88: lambda *args: f'BUILD_WITH_UNITS({", ".join(map(repr, args))})',
    89: lambda *args: f'BUILTIN_OPCODE({", ".join(map(repr, args))})',
    90: lambda *args: f'BYTE({", ".join(map(repr, args))})',
    91: lambda *args: f'BYTE_UNSIGNED({", ".join(map(repr, args))})',
    92: lambda *args: f'CASE({", ".join(map(repr, args))})',
    93: lambda *args: f'CEILING({", ".join(map(repr, args))})',
    94: lambda *args: f'CHAR({", ".join(map(repr, args))})',
    95: lambda *args: f'CLASS({", ".join(map(repr, args))})',
    96: lambda *args: f'FCLOSE({", ".join(map(repr, args))})',
    97: lambda *args: f'CMPLX({", ".join(map(repr, args))})',
    98: lambda *args: f'COMMA({", ".join(map(repr, args))})',
    99: lambda *args: f'COMPILE({", ".join(map(repr, args))})',
    100: lambda *args: f'COMPLETION_OF({", ".join(map(repr, args))})',
    101: lambda *args: f'CONCAT({", ".join(map(repr, args))})',
    102: lambda *args: f'CONDITIONAL({", ".join(map(repr, args))})',
    103: lambda *args: f'CONJG({", ".join(map(repr, args))})',
    104: lambda: 'CONTINUE',
    105: lambda *args: f'CONVOLVE({", ".join(map(repr, args))})',
    106: lambda *args: f'COS({", ".join(map(repr, args))})',
    107: lambda *args: f'COSD({", ".join(map(repr, args))})',
    108: lambda *args: f'COSH({", ".join(map(repr, args))})',
    109: lambda *args: f'COUNT({", ".join(map(repr, args))})',
    110: lambda *args: f'CSHIFT({", ".join(map(repr, args))})',
    111: lambda *args: f'CVT({", ".join(map(repr, args))})',
    112: lambda *args: f'DATA({", ".join(map(repr, args))})',
    113: lambda *args: f'DATE_AND_TIME({", ".join(map(repr, args))})',
    114: lambda *args: f'DATE_TIME({", ".join(map(repr, args))})',
    115: lambda *args: f'DBLE({", ".join(map(repr, args))})',
    116: lambda *args: f'DEALLOCATE({", ".join(map(repr, args))})',
    117: lambda *args: f'DEBUG({", ".join(map(repr, args))})',
    118: lambda *args: f'DECODE({", ".join(map(repr, args))})',
    119: lambda *args: f'DECOMPILE({", ".join(map(repr, args))})',
    120: lambda *args: f'DECOMPRESS({", ".join(map(repr, args))})',
    121: lambda *args: f'DEFAULT({", ".join(map(repr, args))})',
    122: lambda *args: f'DERIVATIVE({", ".join(map(repr, args))})',
    123: lambda *args: f'DESCR({", ".join(map(repr, args))})',
    124: lambda *args: f'DIAGONAL({", ".join(map(repr, args))})',
    125: lambda *args: f'DIGITS({", ".join(map(repr, args))})',
    126: lambda *args: f'DIM({", ".join(map(repr, args))})',
    127: lambda *args: f'DIM_OF({", ".join(map(repr, args))})',
    128: lambda *args: f'DISPATCH_OF({", ".join(map(repr, args))})',
    129: lambda a,b: f"({repr(a)} / {repr(b)})",
    130: lambda *args: f'LBOUND({", ".join(map(repr, args))})',
    131: lambda *args: f'DO({", ".join(map(repr, args))})',
    132: lambda *args: f'DOT_PRODUCT({", ".join(map(repr, args))})',
    133: lambda *args: f'DPROD({", ".join(map(repr, args))})',
    134: lambda *args: f'DSCPTR({", ".join(map(repr, args))})',
    135: lambda *args: f'SHAPE({", ".join(map(repr, args))})',
    136: lambda *args: f'SIZE({", ".join(map(repr, args))})',
    137: lambda *args: f'KIND({", ".join(map(repr, args))})',
    138: lambda *args: f'UBOUND({", ".join(map(repr, args))})',
    139: lambda *args: f'D_COMPLEX({", ".join(map(repr, args))})',
    140: lambda *args: f'D_FLOAT({", ".join(map(repr, args))})',
    141: lambda *args: f'RANGE({", ".join(map(repr, args))})',
    142: lambda *args: f'PRECISION({", ".join(map(repr, args))})',
    143: lambda *args: f'ELBOUND({", ".join(map(repr, args))})',
    144: lambda: 'ELSE',
    145: lambda: 'ELSEWHERE',
    146: lambda *args: f'ENCODE({", ".join(map(repr, args))})',
    147: lambda *args: f'ENDFILE({", ".join(map(repr, args))})',
    148: lambda *args: f'END_OF({", ".join(map(repr, args))})',
    149: lambda *args: f'EOSHIFT({", ".join(map(repr, args))})',
    150: lambda *args: f'EPSILON({", ".join(map(repr, args))})',
    151: lambda *args: f'EQ({", ".join(map(repr, args))})',
    152: lambda *args: f'EQUALS({", ".join(map(repr, args))})',
    153: lambda *args: f'EQUALS_FIRST({", ".join(map(repr, args))})',
    154: lambda *args: f'EQV({", ".join(map(repr, args))})',
    155: lambda *args: f'ESHAPE({", ".join(map(repr, args))})',
    156: lambda *args: f'ESIZE({", ".join(map(repr, args))})',
    157: lambda *args: f'EUBOUND({", ".join(map(repr, args))})',
    158: lambda *args: f'EVALUATE({", ".join(map(repr, args))})',
    159: lambda *args: f'EXECUTE({", ".join(map(repr, args))})',
    160: lambda *args: f'EXP({", ".join(map(repr, args))})',
    161: lambda *args: f'EXPONENT({", ".join(map(repr, args))})',
    162: lambda _,func,*args: f"{func.data()}({', '.join(map(repr, args))})",
    163: lambda *args: f'FFT({", ".join(map(repr, args))})',
    164: lambda *args: f'FIRSTLOC({", ".join(map(repr, args))})',
    165: lambda *args: f'FIT({", ".join(map(repr, args))})',
    166: lambda *args: f'FIX_ROPRAND({", ".join(map(repr, args))})',
    167: lambda *args: f'FLOAT({", ".join(map(repr, args))})',
    168: lambda *args: f'FLOOR({", ".join(map(repr, args))})',
    169: lambda *args: f'FOR({", ".join(map(repr, args))})',
    170: lambda *args: f'FRACTION({", ".join(map(repr, args))})',
    171: lambda *args: f'FUN({", ".join(map(repr, args))})',
    172: lambda *args: f'F_COMPLEX({", ".join(map(repr, args))})',
    173: lambda *args: f'F_FLOAT({", ".join(map(repr, args))})',
    174: lambda *args: f'GE({", ".join(map(repr, args))})',
    175: lambda *args: f'GETNCI({", ".join(map(repr, args))})',
    176: lambda *args: f'GOTO({", ".join(map(repr, args))})',
    177: lambda *args: f'GT({", ".join(map(repr, args))})',
    178: lambda *args: f'G_COMPLEX({", ".join(map(repr, args))})',
    179: lambda *args: f'G_FLOAT({", ".join(map(repr, args))})',
    180: lambda *args: f'HELP_OF({", ".join(map(repr, args))})',
    181: lambda *args: f'HUGE({", ".join(map(repr, args))})',
    182: lambda *args: f'H_COMPLEX({", ".join(map(repr, args))})',
    183: lambda *args: f'H_FLOAT({", ".join(map(repr, args))})',
    184: lambda *args: f'IACHAR({", ".join(map(repr, args))})',
    185: lambda *args: f'IAND({", ".join(map(repr, args))})',
    186: lambda *args: f'IAND_NOT({", ".join(map(repr, args))})',
    187: lambda *args: f'ICHAR({", ".join(map(repr, args))})',
    188: lambda *args: f'IDENT_OF({", ".join(map(repr, args))})',
    189: lambda *args: f'IF({", ".join(map(repr, args))})',
    190: lambda *args: f'IF_ERROR({", ".join(map(repr, args))})',
    191: lambda *args: f'IMAGE_OF({", ".join(map(repr, args))})',
    192: lambda *args: f'IN({", ".join(map(repr, args))})',
    193: lambda *args: f'INAND({", ".join(map(repr, args))})',
    194: lambda *args: f'INAND_NOT({", ".join(map(repr, args))})',
    195: lambda *args: f'INDEX({", ".join(map(repr, args))})',
    196: lambda *args: f'INOR({", ".join(map(repr, args))})',
    197: lambda *args: f'INOR_NOT({", ".join(map(repr, args))})',
    198: lambda *args: f'INOT({", ".join(map(repr, args))})',
    199: lambda *args: f'INOUT({", ".join(map(repr, args))})',
    200: lambda *args: f'INQUIRE({", ".join(map(repr, args))})',
    201: lambda *args: f'INT({", ".join(map(repr, args))})',
    202: lambda *args: f'INTEGRAL({", ".join(map(repr, args))})',
    203: lambda *args: f'INTERPOL({", ".join(map(repr, args))})',
    204: lambda *args: f'INTERSECT({", ".join(map(repr, args))})',
    205: lambda *args: f'INT_UNSIGNED({", ".join(map(repr, args))})',
    206: lambda *args: f'INVERSE({", ".join(map(repr, args))})',
    207: lambda *args: f'IOR({", ".join(map(repr, args))})',
    208: lambda *args: f'IOR_NOT({", ".join(map(repr, args))})',
    209: lambda *args: f'IS_IN({", ".join(map(repr, args))})',
    210: lambda *args: f'IEOR({", ".join(map(repr, args))})',
    211: lambda *args: f'IEOR_NOT({", ".join(map(repr, args))})',
    212: lambda *args: f'LABEL({", ".join(map(repr, args))})',
    213: lambda *args: f'LAMINATE({", ".join(map(repr, args))})',
    214: lambda *args: f'LANGUAGE_OF({", ".join(map(repr, args))})',
    215: lambda *args: f'LASTLOC({", ".join(map(repr, args))})',
    216: lambda *args: f'LE({", ".join(map(repr, args))})',
    217: lambda *args: f'LEN({", ".join(map(repr, args))})',
    218: lambda *args: f'LEN_TRIM({", ".join(map(repr, args))})',
    219: lambda *args: f'LGE({", ".join(map(repr, args))})',
    220: lambda *args: f'LGT({", ".join(map(repr, args))})',
    221: lambda *args: f'LLE({", ".join(map(repr, args))})',
    222: lambda *args: f'LLT({", ".join(map(repr, args))})',
    223: lambda *args: f'LOG({", ".join(map(repr, args))})',
    224: lambda *args: f'LOG10({", ".join(map(repr, args))})',
    225: lambda *args: f'LOG2({", ".join(map(repr, args))})',
    226: lambda *args: f'LOGICAL({", ".join(map(repr, args))})',
    227: lambda *args: f'LONG({", ".join(map(repr, args))})',
    228: lambda *args: f'LONG_UNSIGNED({", ".join(map(repr, args))})',
    229: lambda *args: f'LT({", ".join(map(repr, args))})',
    230: lambda *args: f'MATMUL({", ".join(map(repr, args))})',
    231: lambda *args: f'MAT_ROT({", ".join(map(repr, args))})',
    232: lambda *args: f'MAT_ROT_INT({", ".join(map(repr, args))})',
    233: lambda *args: f'MAX({", ".join(map(repr, args))})',
    234: lambda *args: f'MAXEXPONENT({", ".join(map(repr, args))})',
    235: lambda *args: f'MAXLOC({", ".join(map(repr, args))})',
    236: lambda *args: f'MAXVAL({", ".join(map(repr, args))})',
    237: lambda *args: f'MEAN({", ".join(map(repr, args))})',
    238: lambda *args: f'MEDIAN({", ".join(map(repr, args))})',
    239: lambda *args: f'MERGE({", ".join(map(repr, args))})',
    240: lambda *args: f'METHOD_OF({", ".join(map(repr, args))})',
    241: lambda *args: f'MIN({", ".join(map(repr, args))})',
    242: lambda *args: f'MINEXPONENT({", ".join(map(repr, args))})',
    243: lambda *args: f'MINLOC({", ".join(map(repr, args))})',
    244: lambda *args: f'MINVAL({", ".join(map(repr, args))})',
    245: lambda *args: f'MOD({", ".join(map(repr, args))})',
    246: lambda *args: f'MODEL_OF({", ".join(map(repr, args))})',
    247: lambda a,b: f"({repr(a)} * {repr(b)})",
    248: lambda *args: f'NAME_OF({", ".join(map(repr, args))})',
    249: lambda *args: f'NAND({", ".join(map(repr, args))})',
    250: lambda *args: f'NAND_NOT({", ".join(map(repr, args))})',
    251: lambda *args: f'NDESC({", ".join(map(repr, args))})',
    252: lambda *args: f'NE({", ".join(map(repr, args))})',
    253: lambda *args: f'NEAREST({", ".join(map(repr, args))})',
    254: lambda *args: f'NEQV({", ".join(map(repr, args))})',
    255: lambda *args: f'NINT({", ".join(map(repr, args))})',
    256: lambda *args: f'NOR({", ".join(map(repr, args))})',
    257: lambda *args: f'NOR_NOT({", ".join(map(repr, args))})',
    258: lambda *args: f'NOT({", ".join(map(repr, args))})',
    259: lambda *args: f'OBJECT_OF({", ".join(map(repr, args))})',
    260: lambda *args: f'OCTAWORD({", ".join(map(repr, args))})',
    261: lambda *args: f'OCTAWORD_UNSIGNED({", ".join(map(repr, args))})',
    262: lambda *args: f'ON_ERROR({", ".join(map(repr, args))})',
    263: lambda *args: f'OPCODE_BUILTIN({", ".join(map(repr, args))})',
    264: lambda *args: f'OPCODE_STRING({", ".join(map(repr, args))})',
    265: lambda *args: f'FOPEN({", ".join(map(repr, args))})',
    266: lambda *args: f'OPTIONAL({", ".join(map(repr, args))})',
    267: lambda *args: f'OR({", ".join(map(repr, args))})',
    268: lambda *args: f'OR_NOT({", ".join(map(repr, args))})',
    269: lambda *args: f'OUT({", ".join(map(repr, args))})',
    270: lambda *args: f'PACK({", ".join(map(repr, args))})',
    271: lambda *args: f'PHASE_OF({", ".join(map(repr, args))})',
    272: lambda *args: f'POST_DEC({", ".join(map(repr, args))})',
    273: lambda *args: f'POST_INC({", ".join(map(repr, args))})',
    274: lambda *args: f'POWER({", ".join(map(repr, args))})',
    275: lambda *args: f'PRESENT({", ".join(map(repr, args))})',
    276: lambda *args: f'PRE_DEC({", ".join(map(repr, args))})',
    277: lambda *args: f'PRE_INC({", ".join(map(repr, args))})',
    278: lambda *args: f'PRIVATE({", ".join(map(repr, args))})',
    279: lambda *args: f'PROCEDURE_OF({", ".join(map(repr, args))})',
    280: lambda *args: f'PRODUCT({", ".join(map(repr, args))})',
    281: lambda *args: f'PROGRAM_OF({", ".join(map(repr, args))})',
    282: lambda *args: f'PROJECT({", ".join(map(repr, args))})',
    283: lambda *args: f'PROMOTE({", ".join(map(repr, args))})',
    284: lambda *args: f'PUBLIC({", ".join(map(repr, args))})',
    285: lambda *args: f'QUADWORD({", ".join(map(repr, args))})',
    286: lambda *args: f'QUADWORD_UNSIGNED({", ".join(map(repr, args))})',
    287: lambda *args: f'QUALIFIERS_OF({", ".join(map(repr, args))})',
    288: lambda *args: f'RADIX({", ".join(map(repr, args))})',
    289: lambda *args: f'RAMP({", ".join(map(repr, args))})',
    290: lambda *args: f'RANDOM({", ".join(map(repr, args))})',
    291: lambda *args: f'RANDOM_SEED({", ".join(map(repr, args))})',
    292: lambda *args: f'DTYPE_RANGE({", ".join(map(repr, args))})',
    293: lambda *args: f'RANK({", ".join(map(repr, args))})',
    294: lambda *args: f'RAW_OF({", ".join(map(repr, args))})',
    295: lambda *args: f'READ({", ".join(map(repr, args))})',
    296: lambda *args: f'REAL({", ".join(map(repr, args))})',
    297: lambda *args: f'REBIN({", ".join(map(repr, args))})',
    298: lambda *args: f'REF({", ".join(map(repr, args))})',
    299: lambda *args: f'REPEAT({", ".join(map(repr, args))})',
    300: lambda *args: f'REPLICATE({", ".join(map(repr, args))})',
    301: lambda *args: f'RESHAPE({", ".join(map(repr, args))})',
    302: lambda *args: f'RETURN({", ".join(map(repr, args))})',
    303: lambda *args: f'REWIND({", ".join(map(repr, args))})',
    304: lambda *args: f'RMS({", ".join(map(repr, args))})',
    305: lambda *args: f'ROUTINE_OF({", ".join(map(repr, args))})',
    306: lambda *args: f'RRSPACING({", ".join(map(repr, args))})',
    307: lambda *args: f'SCALE({", ".join(map(repr, args))})',
    308: lambda *args: f'SCAN({", ".join(map(repr, args))})',
    309: lambda *args: f'FSEEK({", ".join(map(repr, args))})',
    310: lambda *args: f'SET_EXPONENT({", ".join(map(repr, args))})',
    311: lambda *args: f'SET_RANGE({", ".join(map(repr, args))})',
    312: lambda *args: f'ISHFT({", ".join(map(repr, args))})',
    313: lambda *args: f'ISHFTC({", ".join(map(repr, args))})',
    314: lambda a,b: f"({repr(a)} << {repr(b)})",
    315: lambda a,b: f"({repr(a)} >> {repr(b)})",
    316: lambda *args: f'SIGN({", ".join(map(repr, args))})',
    317: lambda *args: f'SIGNED({", ".join(map(repr, args))})',
    318: lambda *args: f'SIN({", ".join(map(repr, args))})',
    319: lambda *args: f'SIND({", ".join(map(repr, args))})',
    320: lambda *args: f'SINH({", ".join(map(repr, args))})',
    321: lambda *args: f'SIZEOF({", ".join(map(repr, args))})',
    322: lambda *args: f'SLOPE_OF({", ".join(map(repr, args))})',
    323: lambda *args: f'SMOOTH({", ".join(map(repr, args))})',
    324: lambda *args: f'SOLVE({", ".join(map(repr, args))})',
    325: lambda *args: f'SORTVAL({", ".join(map(repr, args))})',
    326: lambda *args: f'SPACING({", ".join(map(repr, args))})',
    327: lambda *args: f'SPAWN({", ".join(map(repr, args))})',
    328: lambda *args: f'SPREAD({", ".join(map(repr, args))})',
    329: lambda *args: f'SQRT({", ".join(map(repr, args))})',
    330: lambda *args: f'SQUARE({", ".join(map(repr, args))})',
    331: lambda *args: f'STATEMENT({", ".join(map(repr, args))})',
    332: lambda *args: f'STD_DEV({", ".join(map(repr, args))})',
    333: lambda *args: f'STRING({", ".join(map(repr, args))})',
    334: lambda *args: f'STRING_OPCODE({", ".join(map(repr, args))})',
    335: lambda *args: f'SUBSCRIPT({", ".join(map(repr, args))})',
    336: lambda a,b: f"({repr(a)} - {repr(b)})",
    337: lambda *args: f'SUM({", ".join(map(repr, args))})',
    338: lambda *args: f'SWITCH({", ".join(map(repr, args))})',
    339: lambda *args: f'SYSTEM_CLOCK({", ".join(map(repr, args))})',
    340: lambda *args: f'TAN({", ".join(map(repr, args))})',
    341: lambda *args: f'TAND({", ".join(map(repr, args))})',
    342: lambda *args: f'TANH({", ".join(map(repr, args))})',
    343: lambda *args: f'TASK_OF({", ".join(map(repr, args))})',
    344: lambda *args: f'TEXT({", ".join(map(repr, args))})',
    345: lambda *args: f'TIME_OUT_OF({", ".join(map(repr, args))})',
    346: lambda *args: f'TINY({", ".join(map(repr, args))})',
    347: lambda *args: f'TRANSFER({", ".join(map(repr, args))})',
    348: lambda *args: f'TRANSPOSE_({", ".join(map(repr, args))})',
    349: lambda *args: f'TRIM({", ".join(map(repr, args))})',
    350: lambda *args: f'UNARY_MINUS({", ".join(map(repr, args))})',
    351: lambda *args: f'UNARY_PLUS({", ".join(map(repr, args))})',
    352: lambda *args: f'UNION({", ".join(map(repr, args))})',
    353: lambda *args: f'UNITS({", ".join(map(repr, args))})',
    354: lambda *args: f'UNITS_OF({", ".join(map(repr, args))})',
    355: lambda *args: f'UNPACK({", ".join(map(repr, args))})',
    356: lambda *args: f'UNSIGNED({", ".join(map(repr, args))})',
    357: lambda *args: f'VAL({", ".join(map(repr, args))})',
    358: lambda *args: f'VALIDATION_OF({", ".join(map(repr, args))})',
    359: lambda *args: f'VALUE_OF({", ".join(map(repr, args))})',
    360: lambda *args: f'VAR({", ".join(map(repr, args))})',
    361: lambda *args: f'VECTOR({", ".join(map(repr, args))})',
    362: lambda *args: f'VERIFY({", ".join(map(repr, args))})',
    363: lambda *args: f'WAIT({", ".join(map(repr, args))})',
    364: lambda *args: f'WHEN_OF({", ".join(map(repr, args))})',
    365: lambda *args: f'WHERE({", ".join(map(repr, args))})',
    366: lambda *args: f'WHILE({", ".join(map(repr, args))})',
    367: lambda *args: f'WINDOW_OF({", ".join(map(repr, args))})',
    368: lambda *args: f'WORD({", ".join(map(repr, args))})',
    369: lambda *args: f'WORD_UNSIGNED({", ".join(map(repr, args))})',
    370: lambda *args: f'WRITE({", ".join(map(repr, args))})',
    371: lambda *args: f'ZERO({", ".join(map(repr, args))})',
    372: lambda: '$2PI',
    373: lambda: '$NARG',
    374: lambda *args: f'ELEMENT({", ".join(map(repr, args))})',
    375: lambda *args: f'RC_DROOP({", ".join(map(repr, args))})',
    376: lambda: 'RESET_PRIVATE',
    377: lambda: 'RESET_PUBLIC',
    378: lambda *args: f'SHOW_PRIVATE({", ".join(map(repr, args))})',
    379: lambda *args: f'SHOW_PUBLIC({", ".join(map(repr, args))})',
    380: lambda *args: f'SHOW_VM({", ".join(map(repr, args))})',
    381: lambda *args: f'TRANSLATE({", ".join(map(repr, args))})',
    382: lambda *args: f'TRANSPOSE_MUL({", ".join(map(repr, args))})',
    383: lambda *args: f'UPCASE({", ".join(map(repr, args))})',
    384: lambda *args: f'USING({", ".join(map(repr, args))})',
    385: lambda *args: f'VALIDATION({", ".join(map(repr, args))})',
    386: lambda: '$DEFAULT',
    387: lambda: '$EXPT',
    388: lambda: '$SHOT',
    389: lambda *args: f'GETDBI({", ".join(map(repr, args))})',
    390: lambda *args: f'CULL({", ".join(map(repr, args))})',
    391: lambda *args: f'EXTEND({", ".join(map(repr, args))})',
    392: lambda *args: f'I_TO_X({", ".join(map(repr, args))})',
    393: lambda *args: f'X_TO_I({", ".join(map(repr, args))})',
    394: lambda *args: f'MAP({", ".join(map(repr, args))})',
    395: lambda *args: f'COMPILE_DEPENDENCY({", ".join(map(repr, args))})',
    396: lambda *args: f'DECOMPILE_DEPENDENCY({", ".join(map(repr, args))})',
    397: lambda *args: f'BUILD_CALL({", ".join(map(repr, args))})',
    398: lambda *args: f'ERRORLOGS_OF({", ".join(map(repr, args))})',
    399: lambda *args: f'PERFORMANCE_OF({", ".join(map(repr, args))})',
    400: lambda *args: f'XD({", ".join(map(repr, args))})',
    401: lambda *args: f'CONDITION_OF({", ".join(map(repr, args))})',
    402: lambda *args: f'SORT({", ".join(map(repr, args))})',
    403: lambda: '$THIS',
    404: lambda *args: f'DATA_WITH_UNITS({", ".join(map(repr, args))})',
    405: lambda: '$ATM',
    406: lambda: '$EPSILON0',
    407: lambda: '$GN',
    408: lambda: '$MU0',
    409: lambda *args: f'EXTRACT({", ".join(map(repr, args))})',
    410: lambda *args: f'FINITE({", ".join(map(repr, args))})',
    411: lambda *args: f'BIT_SIZE({", ".join(map(repr, args))})',
    412: lambda a,b: f"({repr(a)} % {repr(b)})",
    413: lambda *args: f'SELECTED_INT_KIND({", ".join(map(repr, args))})',
    414: lambda *args: f'SELECTED_REAL_KIND({", ".join(map(repr, args))})',
    415: lambda *args: f'DSQL({", ".join(map(repr, args))})',
    416: lambda *args: f'ISQL({", ".join(map(repr, args))})',
    417: lambda *args: f'FTELL({", ".join(map(repr, args))})',
    418: lambda *args: f'MAKE_ACTION({", ".join(map(repr, args))})',
    419: lambda *args: f'MAKE_CONDITION({", ".join(map(repr, args))})',
    420: lambda *args: f'MAKE_CONGLOM({", ".join(map(repr, args))})',
    421: lambda *args: f'MAKE_DEPENDENCY({", ".join(map(repr, args))})',
    422: lambda *args: f'MAKE_DIM({", ".join(map(repr, args))})',
    423: lambda *args: f'MAKE_DISPATCH({", ".join(map(repr, args))})',
    424: lambda *args: f'MAKE_FUNCTION({", ".join(map(repr, args))})',
    425: lambda *args: f'MAKE_METHOD({", ".join(map(repr, args))})',
    426: lambda *args: f'MAKE_PARAM({", ".join(map(repr, args))})',
    427: lambda *args: f'MAKE_PROCEDURE({", ".join(map(repr, args))})',
    428: lambda *args: f'MAKE_PROGRAM({", ".join(map(repr, args))})',
    429: lambda *args: f'MAKE_RANGE({", ".join(map(repr, args))})',
    430: lambda *args: f'MAKE_ROUTINE({", ".join(map(repr, args))})',
    431: lambda *args: f'MAKE_SIGNAL({", ".join(map(repr, args))})',
    432: lambda *args: f'MAKE_WINDOW({", ".join(map(repr, args))})',
    433: lambda *args: f'MAKE_WITH_UNITS({", ".join(map(repr, args))})',
    434: lambda *args: f'MAKE_CALL({", ".join(map(repr, args))})',
    435: lambda *args: f'CLASS_OF({", ".join(map(repr, args))})',
    436: lambda *args: f'DSCPTR_OF({", ".join(map(repr, args))})',
    437: lambda *args: f'KIND_OF({", ".join(map(repr, args))})',
    438: lambda *args: f'NDESC_OF({", ".join(map(repr, args))})',
    439: lambda *args: f'ACCUMULATE({", ".join(map(repr, args))})',
    440: lambda *args: f'MAKE_SLOPE({", ".join(map(repr, args))})',
    441: lambda *args: f'REM({", ".join(map(repr, args))})',
    442: lambda *args: f'COMPLETION_MESSAGE_OF({", ".join(map(repr, args))})',
    443: lambda *args: f'INTERRUPT_OF({", ".join(map(repr, args))})',
    444: lambda: '$SHOTNAME',
    445: lambda *args: f'BUILD_WITH_ERROR({", ".join(map(repr, args))})',
    446: lambda *args: f'ERROR_OF({", ".join(map(repr, args))})',
    447: lambda *args: f'MAKE_WITH_ERROR({", ".join(map(repr, args))})',
    448: lambda *args: f'DO_TASK({", ".join(map(repr, args))})',
    449: lambda *args: f'ISQL_SET({", ".join(map(repr, args))})',
    450: lambda *args: f'FS_FLOAT({", ".join(map(repr, args))})',
    451: lambda *args: f'FS_COMPLEX({", ".join(map(repr, args))})',
    452: lambda *args: f'FT_FLOAT({", ".join(map(repr, args))})',
    453: lambda *args: f'FT_COMPLEX({", ".join(map(repr, args))})',
    454: lambda *args: f'BUILD_OPAQUE({", ".join(map(repr, args))})',
    455: lambda *args: f'MAKE_OPAQUE({", ".join(map(repr, args))})',
    456: lambda *args: f'DICT({", ".join(map(repr, args))})',
    457: lambda *args: f'TUPLE({", ".join(map(repr, args))})',
    458: lambda *args: f'LIST({", ".join(map(repr, args))})',
    459: lambda *args: f'SQUEEZE({", ".join(map(repr, args))})',
    460: lambda *args: f'FLATTEN({", ".join(map(repr, args))})',
}
